package ai.group_template;

import java.util.Map;

import javolution.util.FastMap;

import net.sf.l2j.gameserver.ai.CtrlIntention;
import net.sf.l2j.gameserver.model.L2CharPosition;
import net.sf.l2j.gameserver.model.L2Attackable;
import net.sf.l2j.gameserver.model.actor.instance.L2NpcInstance;
import net.sf.l2j.gameserver.model.actor.instance.L2PcInstance;
import net.sf.l2j.gameserver.network.serverpackets.CreatureSay;
import net.sf.l2j.util.Rnd;

/**
 * flee onAttack (current version is rather fear than retreat)
 * TODO: find a way to check position instead of using a timer to stop fleeing make zones or a list of "basements" for mobs to retreat to.
 */
public class RetreatOnAttack extends L2AttackableAIScript
{
	private static final Map<Integer, Integer[]> MOBSPAWNS = new FastMap<Integer, Integer[]>();
	private static final Map<Integer, String[]> MOBTEXT = new FastMap<Integer, String[]>();
	static
	{
		MOBSPAWNS.put(20058, new Integer[] { 50, 10 }); // Ol Mahum Guard
		MOBSPAWNS.put(20207, new Integer[] { 30, 1 }); // Ol Mahum Guerilla
		MOBSPAWNS.put(20208, new Integer[] { 50, 1 }); // Ol Mahum Raider
		MOBSPAWNS.put(21508, new Integer[] { 30, 100 }); // Splinter Stakato
		MOBSPAWNS.put(21509, new Integer[] { 30, 100 }); // Splinter Stakato Walker
		MOBSPAWNS.put(21510, new Integer[] { 30, 100 }); // Splinter Stakato Soldier
		MOBSPAWNS.put(21511, new Integer[] { 30, 100 }); // Splinter Stakato Drone
		MOBSPAWNS.put(21512, new Integer[] { 30, 100 }); // Splinter Stakato Drone
		MOBSPAWNS.put(21513, new Integer[] { 30, 100 }); // Needle Stakato
		MOBSPAWNS.put(21514, new Integer[] { 30, 100 }); // Needle Stakato Worker
		MOBSPAWNS.put(21515, new Integer[] { 30, 100 }); // Needle Stakato Soldier
		MOBSPAWNS.put(20158, new Integer[] { 30, 1 }); //  Medusa
		MOBSPAWNS.put(20497, new Integer[] { 30, 80 }); // Turek Orc Skirmisher
		MOBSPAWNS.put(20500, new Integer[] { 30, 70 }); // Turek Orc Sentinel
	}
	static
	{
		MOBTEXT.put(20058, new String[] { "I'll be back.", "You are stronger than expected." });
		MOBTEXT.put(20207, new String[] { "Invader!", "I'll give you ten million adenas if you let me live!", "I will definitely repay this humiliation!", "There is no reason for you to kill me! I have nothing you need!", "Tactical retreat!", "Some day you will pay!", "Stop hitting!", "We shall see about that!", "I'll kill you next time!", "I definitely kill you next time!" });
		MOBTEXT.put(20208, new String[] { "Retreat!", "Mass fleeing!", "Surrender!", "I'll be back!", "Oh! How strong!" });
		MOBTEXT.put(20158, new String[] { "It's a good day to die! Welcome to hell, maggot!", "Some day I will kick ya ass!", "Don't cry, I will come back!", "I'll be back!", "Oh! Oh! Oh! Go your'e way! I die my Lord!!!" });
		MOBTEXT.put(20497, new String[] { "Oh! How Strong!", "We shall see about that", "I'll definitely kill you next time" });
		MOBTEXT.put(20500, new String[] { "Oh! How Strong!", "We shall see about that", "I'll definitely kill you next time" });
	}

	public RetreatOnAttack(int questId, String name, String descr)
	{
		super(questId, name, descr);
		for (int id : MOBSPAWNS.keySet())
			super.addAttackId(id);
	}

	public String onAdvEvent(String event, L2NpcInstance npc, L2PcInstance player)
	{
		if (event.equalsIgnoreCase("Retreat"))
		{
			npc.stopFear(null);
			((L2Attackable) npc).addDamageHate(player, 0, 100);
			npc.getAI().setIntention(CtrlIntention.AI_INTENTION_ATTACK, player);
		}
		return super.onAdvEvent(event, npc, player);
	}

	public String onAttack(L2NpcInstance npc, L2PcInstance attacker, int damage, boolean isPet)
	{
		double currentHp = npc.getCurrentHp();
		int maxHp = npc.getMaxHp();
		int npcId = npc.getNpcId();
		if (MOBSPAWNS.containsKey(npcId))
		{
			Integer[] tmp = MOBSPAWNS.get(npcId);
			if ((currentHp <= (maxHp * tmp[0] / 100)) && Rnd.get(100) <= tmp[1])
			{
				String[] tmptxt = MOBTEXT.get(npcId);
				if (MOBTEXT.containsKey(npcId))
					npc.broadcastPacket(new CreatureSay(npc.getObjectId(), 0, npc.getName(), (tmptxt[Rnd.get(tmptxt.length)])));
				int posX = npc.getX();
				int posY = npc.getY();
				int posZ = npc.getZ();
				int signX = -500;
				int signY = -500;
				if (posX > attacker.getX())
					signX = 500;
				if (posY > attacker.getY())
					signY = 500;
				posX = posX + signX;
				posY = posY + signY;
				npc.startFear();
				npc.setRunning();
				npc.getAI().setIntention(CtrlIntention.AI_INTENTION_MOVE_TO, new L2CharPosition(posX, posY, posZ, 0));
				startQuestTimer("Retreat", 10000, npc, attacker);
			}
		}
		return super.onAttack(npc, attacker, damage, isPet);
	}

	public static void main(String[] args)
	{
		new RetreatOnAttack(-1, "RetreatOnAttack", "ai");
	}
}