package ai.group_template;

import java.util.Map;

import javolution.util.FastMap;

import net.sf.l2j.Config;
import net.sf.l2j.gameserver.ai.CtrlIntention;
import net.sf.l2j.gameserver.model.L2Attackable;
import net.sf.l2j.gameserver.model.actor.instance.L2NpcInstance;
import net.sf.l2j.gameserver.model.actor.instance.L2PcInstance;
import net.sf.l2j.gameserver.model.quest.Quest;
import net.sf.l2j.util.Rnd;

/**
 * @update S-Project
 */
public class Splendor extends Quest
{
	private static final int[] NPC =
	{
		21521, 21524, 21527, 21537, 21539,
		21522, 21525, 21528, 21538, 21540
	};
	private static boolean AlwaysSpawn;

	private static Map<Integer, int[]> SplendorId = new FastMap<Integer, int[]>();

	public Splendor(int questId, String name, String descr)
	{
		super(questId, name, descr);

		AlwaysSpawn = false;
		for(int NPC_ID: NPC)
		{
			addEventId(NPC_ID, Quest.QuestEventType.ON_ATTACK);
			addEventId(NPC_ID, Quest.QuestEventType.ON_KILL);
		}
		// Npc:[NewNpc,% for chance by shot,ModeSpawn]
		// Modespawn 1=> delete and spawn the news npc
		// Modespawn 2=> just add 1 spawn the news npc
		// if Quest_Drop = 5 => 25% by shot to change mob
		SplendorId.put(21521, new int[] {21522,5,1}); // Claw of Splendor
		SplendorId.put(21524, new int[] {21525,5,1}); // Blade of Splendor
		SplendorId.put(21527, new int[] {21528,5,1}); // Anger of Splendor
		SplendorId.put(21537, new int[] {21538,5,1}); // Fang of Splendor
		SplendorId.put(21539, new int[] {21540,100,2});
	}

	@Override
	public String onAttack (L2NpcInstance npc, L2PcInstance attacker, int damage, boolean isPet)
	{
		int npcId = npc.getNpcId();
		int NewMob = SplendorId.get(npcId)[0];
		int chance = SplendorId.get(npcId)[1];
		int ModeSpawn = SplendorId.get(npcId)[2];
		if(Rnd.get(100) <= chance * Config.RATE_DROP_QUEST)
			if(SplendorId.containsKey(npcId))
				if(ModeSpawn == 1)
				{
					npc.deleteMe();
					L2Attackable newNpc = (L2Attackable) addSpawn(NewMob, npc);
					newNpc.addDamageHate(attacker,0,999);
					newNpc.getAI().setIntention(CtrlIntention.AI_INTENTION_ATTACK, attacker);
				}
				else if(AlwaysSpawn)
					return super.onAttack(npc, attacker, damage, isPet);
				else if(ModeSpawn == 2)
				{
					AlwaysSpawn = true;
					L2Attackable newNpc1 = (L2Attackable) addSpawn(NewMob, npc);
					newNpc1.addDamageHate(attacker,0,999);
					newNpc1.getAI().setIntention(CtrlIntention.AI_INTENTION_ATTACK, attacker);
				}

		return super.onAttack(npc, attacker, damage, isPet);
	}

	@Override
	public String onKill(L2NpcInstance npc, L2PcInstance killer, boolean isPet)
	{
		int npcId = npc.getNpcId();
		int NewMob = SplendorId.get(npcId)[0];
		int chance = SplendorId.get(npcId)[1];
		int ModeSpawn = SplendorId.get(npcId)[2];
		if(SplendorId.containsKey(npcId))
			if(ModeSpawn == 2)
				AlwaysSpawn = false;

		return super.onKill(npc,killer,isPet);
	}
}