package net.sf.Event;

import net.sf.l2j.Config;
import net.sf.l2j.gameserver.Announcements;
import net.sf.l2j.gameserver.model.actor.instance.*;
import net.sf.l2j.gameserver.model.quest.Quest;

import java.util.Calendar;

/**
 * @update S-Project
 */
public class CrazyEnchant extends Quest
{
   private static final boolean ANNOUNCE = Config.Announce_CrazyEvent; // set it true/false for Announce
   private static final int PERCENT = Config.Enchant_Chance_CrazyEvent; // Percent from 100%
   private static final int DURATION = Config.Duration_CrazyEvent; // Duration
   private static final int DELAY = Config.Delay_Start_CrazyEvent; // delay when server started
 
   private static final Byte START_HOUR = Config.Allow_EnebleTime_CrazyEvent; // 0 to Disable
                                                                    
   private static final Byte START_MINUTE = Config.Allow_EnebleMinutes_CrazyEvent;
 
   private static final int FREQUENCY = Config.Return_Time_CrazyEvent; // Rerun time between events in Minutes
 
   public CrazyEnchant(int questId, String name, String descr)
   {
      super(questId, name, descr);
 
      if (loadGlobalQuestVar("enchantrate") == "")
      {
         if (START_HOUR > 0)
            startQuestTimer("START", setStartTime(START_HOUR, START_MINUTE), null, null);
         else
            startQuestTimer("START", DELAY * 60000, null, null);
      }
      else // in case your server crashed before restoring rates...
         startQuestTimer("RESTORE", 60000, null, null);
   }
 
   private long setStartTime(Byte hour, Byte minute)
   {
      Calendar starttime = Calendar.getInstance();
      if (hour > 0 && hour < 24)
         starttime.set(Calendar.HOUR_OF_DAY, hour);
      else
          _log.info("Шалена заточка: неправильное время для дня.");
      if (minute > 0 && minute < 60)
         starttime.set(Calendar.MINUTE, minute);
      else if (minute > 60)
         starttime.set(Calendar.MINUTE, minute % 60);
      if ((starttime.getTimeInMillis() - Calendar.getInstance().getTimeInMillis()) > 0)
         return (starttime.getTimeInMillis() - Calendar.getInstance().getTimeInMillis());
      else return 60000L;
   }
 
   @Override
public String onAdvEvent(String event, L2NpcInstance npc, L2PcInstance player)
   {
      if (event.equalsIgnoreCase("START"))
      {
         saveGlobalQuestVar("armorenchantrate", String.valueOf(Config.ENCHANT_CHANCE_ARMOR));
         Config.ENCHANT_CHANCE_ARMOR = PERCENT + Config.ENCHANT_CHANCE_ARMOR;
         saveGlobalQuestVar("weaponenchantrate", String.valueOf(Config.ENCHANT_CHANCE_WEAPON));
         Config.ENCHANT_CHANCE_WEAPON = PERCENT + Config.ENCHANT_CHANCE_WEAPON;
         saveGlobalQuestVar("jewelryenchantrate", String.valueOf(Config.ENCHANT_CHANCE_JEWELRY));
         Config.ENCHANT_CHANCE_JEWELRY = PERCENT + Config.ENCHANT_CHANCE_JEWELRY;
         if (ANNOUNCE)
         Announcements.getInstance().announceToAll("Евент -Шалена заточка- Запустился! В течение " + DURATION + " следующих минут шанс заточки увеличен на "+ PERCENT +"!");
         startQuestTimer("RESTORE", DURATION * 60000, null, null);
         _log.info("Шалена заточка: шанс изминен. восстановления прежнего шанса через " + DURATION + " минут.");
      }
 
      if (event.equalsIgnoreCase("RESTORE"))
      {
         Config.ENCHANT_CHANCE_ARMOR = Integer.valueOf(loadGlobalQuestVar("armorenchantrate"));
         Config.ENCHANT_CHANCE_WEAPON = Integer.valueOf(loadGlobalQuestVar("weaponenchantrate"));
         Config.ENCHANT_CHANCE_JEWELRY = Integer.valueOf(loadGlobalQuestVar("jewelryenchantrate"));
         deleteAllGlobalQuestVars();
         _log.info("Шалена заточка: шанс точки урегулирован.");
 
         if (ANNOUNCE)
            Announcements.getInstance().announceToAll("Шалена заточка: шанс заточки востановлен, Поздравляем всех кто успел заточится!");
         if (START_HOUR > 0)
            startQuestTimer("START", setStartTime(START_HOUR, START_MINUTE), null, null);
         else if (FREQUENCY > 0)
            startQuestTimer("START", FREQUENCY * 60000, null, null);
      }
      return "";
   }
 
   public static void main(String[] args)
   {
      new CrazyEnchant(-1, "CrazyEnchant", "events");
   }
}