package net.sf.PowerPack.vote;

import javolution.util.*;
import net.sf.PowerPack.PowerPackConfig;
import net.sf.l2j.L2DatabaseFactory;
import net.sf.l2j.gameserver.ThreadPoolManager;
import net.sf.l2j.gameserver.datatables.ItemTable;
import net.sf.l2j.gameserver.idfactory.IdFactory;
import net.sf.l2j.gameserver.model.*;
import net.sf.l2j.gameserver.model.actor.instance.L2PcInstance;
import net.sf.l2j.gameserver.network.SystemMessageId;
import net.sf.l2j.gameserver.network.serverpackets.*;
import org.apache.commons.logging.*;

import java.io.*;
import java.net.*;
import java.sql.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.concurrent.ScheduledFuture;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author TheFosT
 * @update S-Project
 */
public class L2TopDeamon
{
	 protected static final Log _log = LogFactory.getLog(L2TopDeamon.class);
	    private Pattern _webPattern;
	    private Pattern _smsPattern;

	    private static final DateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	    private static L2TopDeamon _instance = null;
	    private static FastMap<Integer, Long> _voteDateCache = new FastMap<Integer, Long>().setShared(true);  // ???: charObjId=>lastVoteDate

	    private static String USERAGENT = "Mozilla/5.0 (%Platform%; %Version%; %Architecture%; U) %JVM%/%JVMVer%"
	                .replace("%Platform%", System.getProperty("os.name")).replace("%Version%", System.getProperty("os.version")).replace("%Architecture%", System.getProperty("os.arch"))
	                .replace("%JVM%", System.getProperty("java.vm.name")).replace("%JVMVer%", System.getProperty("java.vm.version"));

	    public static L2TopDeamon getInstance()
	    {
	        if (_instance == null)
	            _instance = new L2TopDeamon();
	        return _instance;
	    }

	    private ScheduledFuture<?> _task;
	    private class Terminator extends Thread
        {
			@Override
			public void run()
            {
				System.out.println("L2TopDeamon: stopped");
				try
                {
					if(L2TopDeamon.getInstance()._task!=null)
                    {
						L2TopDeamon.getInstance()._task.cancel(true);
					}
				}
                catch(Exception e)
				{ 
					_log.info("L2TopDeamon:  Terminator");
				}
			}
		} 
	    
	    private L2TopDeamon()
	    {
	        if (PowerPackConfig.L2Top_DELAY < 1)
	            return;
	        _log.info("L2TopDeamon: Initializing.");
	        _webPattern = Pattern.compile("^([\\d-]+\\s[\\d:]+)\\s+(?:"+PowerPackConfig.L2Top_PREFIX+"-)*([a-zA-Z0-9-]+)$", Pattern.MULTILINE);
	        _smsPattern = Pattern.compile("^([\\d-]+\\s[\\d:]+)\\s+(?:"+PowerPackConfig.L2Top_PREFIX+"-)*([a-zA-Z0-9-]+)\\s+x(\\d{1,2})$", Pattern.MULTILINE);

	        ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new L2TopRuTask(), PowerPackConfig.L2Top_DELAY, PowerPackConfig.L2Top_DELAY);
	    }

	    private class L2TopRuTask implements Runnable
	    {
	        @Override
	        public void run()
	        {
	        	L2TopDeamon.getInstance().tick();
	        }
	    }

	    private FastTable<L2TopRuVote> filterVotes(FastTable<L2TopRuVote> L2TopRuManager)
	    {
	        FastTable<L2TopRuVote> result = new FastTable<L2TopRuVote>();
	        FastMap<String, Integer> chars = new FastMap<String, Integer>(); 
	      
	        Statement stmt = null;
	        ResultSet rset = null;
	        Connection con= null;
	       try
	       	{
              con = L2DatabaseFactory.getInstance().getConnection();
              stmt = con.createStatement();
              rset = stmt.executeQuery("SELECT `obj_Id`,`char_name` FROM `characters`");

              while(rset.next())
                      chars.put(rset.getString("char_name").toLowerCase(), rset.getInt("obj_Id"));

                    stmt.close();
                      rset.close();
                      if(con != null)
	                    {
	                    	con.close();
	                    	con=null;
	                    } 
                      
	       		}
	        
	        catch (Exception ex)
	        {
	            ex.printStackTrace();
	            _log.info("L2TopDeamon:  FastTable Error"+ex.getMessage());
	        }
	        finally
	        {
	        	   
	        }

	        int charObjId = 0;
	        for (L2TopRuVote vote : L2TopRuManager)
	        {
	            if(chars.containsKey(vote.charname)) 
	            {
	                charObjId = chars.get(vote.charname); 
	                if(isRewardReq(charObjId, vote.datetime)) 
	                {
	                    vote.char_obj_id = charObjId;
	                    result.add(vote);
	                }
	            }
	        }
	        
	        con= null;
	        return result;
	    }

	   private boolean isRewardReq(int charObjId, long date)
	   {
	        long lastDate = 0;
	        Connection con = null;
	        PreparedStatement pstmt = null;
	        ResultSet rset = null;

	        if(_voteDateCache.containsKey(charObjId)) 
	        {
	            lastDate = _voteDateCache.get(charObjId); 
	            if(date > lastDate) 
	            {
	                _voteDateCache.put(charObjId, date); 
	                try
	                {
	                	con = L2DatabaseFactory.getInstance().getConnection();
	                    pstmt = con.prepareStatement("REPLACE DELAYED INTO `l2votes`(`obj_Id`,`last_vote`) VALUES (?,?)"); 
	                    pstmt.setInt(1, charObjId);
	                    pstmt.setLong(2, date);
	                    pstmt.executeUpdate();
	                    pstmt.close();
	                    if(con != null)
	                    {
	                    	con.close();
	                    	con=null;
	                    }
	                    
	                }
	                catch (Exception ex)
	                {
	                    ex.printStackTrace();
	                    _log.info("L2TopDeamon:  isRewardReq Error "+ex.getMessage());
	                }
	                return true; 
	            }
	        }
	        else 
	        {
	            try
	            {
	                con = L2DatabaseFactory.getInstance().getConnection();
	                pstmt = con.prepareStatement("SELECT `obj_Id`,`last_vote` FROM `l2votes` WHERE `obj_Id` = ?");
	                pstmt.setInt(1, charObjId);
	                rset = pstmt.executeQuery();
	                if(rset.next()) 
	                {
	                    lastDate = rset.getLong("last_vote");
	                    pstmt.close();
	                    rset.close();
	                    if (date > lastDate) 
	                    {
	                        _voteDateCache.put(charObjId, date); 

	                        try
	                        {
	                            pstmt = con.prepareStatement("REPLACE DELAYED INTO `l2votes`(`obj_Id`,`last_vote`) VALUES (?,?)"); // ? ??????? ? ??
	                            pstmt.setInt(1, charObjId);
	                            pstmt.setLong(2, date);
	                            pstmt.executeUpdate();
	                            pstmt.close();
	                            if(con != null) con.close();
	                        }
	                        catch (Exception ex)
	                        {
	                            ex.printStackTrace();
	                        }
	                        return true; 
	                    }
	                    else // ???
	                        _voteDateCache.put(charObjId, lastDate); 

	                }
	                else 
	                {
	                    pstmt.close();
	                    rset.close();
	                    _voteDateCache.put(charObjId, date); 
	                    try
	                    {
	                        pstmt = con.prepareStatement("REPLACE DELAYED INTO `l2votes`(`obj_Id`,`last_vote`) VALUES (?,?)"); 
	                        pstmt.setInt(1, charObjId);
	                        pstmt.setLong(2, date);
	                        pstmt.executeUpdate();
	                        pstmt.close();
	                        if(con != null)
                            {
	                        	con.close();
	                        	con=null;
	                        }
	                    }
	                    
	                    catch (Exception ex)
	                    {
	                        ex.printStackTrace();
	                        _log.info("L2TopDeamon: REPLACE DELAYED INTO Error"+ex.getMessage());
	                    }

	                    return true; 
	                }
	            }
	            catch (Exception ex)
	            {
	                ex.printStackTrace();
	            }
	            finally
	            {
	            	if(con != null) 
	            		try
	            	{
	            			con.close();
	            			con=null;
	            			
	            	}
                        catch(Exception ex)
                        {
	            		_log.info("L2TopDeamon: REPLACE DELAYED INTO Error"+ex.getMessage());
	            	}
	            }
	        }
	           return false;
	    }

	    private void rewardVotes(FastTable<L2TopRuVote> L2TopRuManager)
	    {
	        for (L2TopRuVote vote : L2TopRuManager)
	        {
	            switch (vote.type)
	            {
	                case WEB:
	                    _log.info("L2TopDeamon: "+vote.toString());
	                    giveItem(vote.char_obj_id, PowerPackConfig.L2Top_WEB_REWARD_ITEMID, PowerPackConfig.L2Top_WEB_REWARD_ITEMCOUNT);

	                    break;
	                case SMS:
	                    _log.info("L2TopDeamon: "+vote.toString());
	                        giveItem(vote.char_obj_id, PowerPackConfig.L2Top_SMS_REWARD_ITEMID,
	                                 (PowerPackConfig.L2Top_SMS_REWARD_VOTE_MULTI ? PowerPackConfig.L2Top_SMS_REWARD_ITEMCOUNT * vote.count : PowerPackConfig.L2Top_SMS_REWARD_ITEMCOUNT));
	                    break;
	            }
	        }
	    }

	    private void giveItem(int charObjId, int itemId, int itemCount)
	    {
	        if (charObjId < 1)
	            return;
	        L2PcInstance player = (L2PcInstance)L2World.getInstance().findObject(charObjId);
	        if (player != null)
	        {
	                L2ItemInstance item = player.getInventory().addItem("L2TopVoting", itemId, itemCount, null, null);
	                InventoryUpdate iu = new InventoryUpdate();
	                iu.addItem(item);
	                player.sendPacket(iu);
	                SystemMessage sm = new SystemMessage(SystemMessageId.YOU_PICKED_UP_S1_S2);
	                sm.addItemName(itemId);
	                sm.addNumber(itemCount);
	                player.sendPacket(sm);
	        }
	        else
	        {
	            L2ItemInstance newItem = new L2ItemInstance(IdFactory.getInstance().getNextId(), ItemTable.getInstance().getTemplate(itemId));
	            newItem.setCount(itemCount);
	            newItem.setOwnerId(charObjId);
	            newItem.setLocation(L2ItemInstance.ItemLocation.INVENTORY);
	            newItem.updateDatabase();
	        }
	    }

	    private FastTable<L2TopRuVote> getAllVotes()
	    {
	        FastTable<L2TopRuVote> result = new FastTable<L2TopRuVote>();
	        try
	        {
	            L2TopRuVote vote;
	            Matcher m = _webPattern.matcher(getPage(PowerPackConfig.L2Top_WEB_VOTE_URL));
	            while(m.find())
	            {
	                vote = new L2TopRuVote(m.group(1), m.group(2));
	                result.add(vote);
	            }

	            m = _smsPattern.matcher(getPage(PowerPackConfig.L2Top_SMS_VOTE_URL));
	            while(m.find())
	            {
	                vote = new L2TopRuVote(m.group(1), m.group(2), m.group(3));
	                result.add(vote);
	            }

	            result.setValueComparator(new L2TopRuVoteComparator());
	        }
	        catch (Exception ex)
	        {
	            ex.printStackTrace();
	        }
	        return result;
	    }

	    private void tick()
	    {
	        _log.info("L2TopDeamon: Rewarding votes");
	        rewardVotes(filterVotes(getAllVotes()));
	    }

	    private String getPage(String uri) throws Exception
	    {
	        URL url = new URL(uri);
	        URLConnection conn = url.openConnection();

	        conn.addRequestProperty("Host", url.getHost());
	        conn.addRequestProperty("Accept", "*/*");
	        conn.addRequestProperty("Connection", "close");
	        conn.addRequestProperty("User-Agent", USERAGENT);

	        BufferedReader in = new BufferedReader(new InputStreamReader(conn.getInputStream(), "cp1251"));  // cp1251 (((
	        StringBuilder builder = new StringBuilder();
	        String line = null;
	        while((line = in.readLine()) != null)
	        {
	            builder.append(line).append("\n");
	        }
	        return builder.toString();
	    }

	    private enum L2TopRuVoteType
	    {
	        WEB,
	        SMS
	    }

	    private class L2TopRuVote
	    {
	        public long datetime;
	        public String charname;
	        public int count;
	        public int char_obj_id = -1;
	        public L2TopRuVoteType type;

	        public L2TopRuVote(String date, String charName, String itemcount) throws Exception
	        {
	            datetime = DATE_FORMAT.parse(date).getTime() / 1000;
	            count = Byte.parseByte(itemcount);
	            charname = charName.toLowerCase();
	            type = L2TopRuVoteType.SMS;
	        }

	        public L2TopRuVote(String date, String charName) throws Exception
	        {
	            datetime = DATE_FORMAT.parse(date).getTime() / 1000;
	            charname = charName.toLowerCase();
	            count = 1;
	            type = L2TopRuVoteType.WEB;
	        }

	        @Override
	        public String toString()
	        {
	            return charname + "-" + count + "[" + char_obj_id + "(" + datetime + "|" + type.name() + ")]";
	        }
	    }

	    private final class L2TopRuVoteComparator<T> extends FastComparator<L2TopRuVote>
	    {
	        @Override
	        public int hashCodeOf(L2TopRuVote o)
	        {
	            return o.hashCode();
	        }

	        @Override
	        public boolean areEqual(L2TopRuVote o, L2TopRuVote o1)
	        {
	            return o.char_obj_id == o1.char_obj_id && o.datetime == o1.datetime;
	        }

	        @Override
	        public int compare(L2TopRuVote o, L2TopRuVote o1)
	        {
	            if (o.datetime == o1.datetime)
	                return 0;
	            if (o.datetime < o1.datetime)
	                return Integer.MIN_VALUE;
	            if (o.datetime > o1.datetime)
	                return Integer.MAX_VALUE;
	            return -1;
	        }
	  }
}