package net.sf.l2j.gameserver;

import net.sf.Event.manager.EventManager;
import net.sf.PowerPack.*;
import net.sf.l2ddt.engine.*;
import net.sf.l2j.*;
import net.sf.l2j.Config;
import net.sf.l2j.gameserver.Geo.GeoData;
import net.sf.l2j.gameserver.Shutdown;
import net.sf.l2j.gameserver.ai.special.manager.*;
import net.sf.l2j.gameserver.cache.*;
import net.sf.l2j.gameserver.communitybbs.Manager.ForumsBBSManager;
import net.sf.l2j.gameserver.datatables.*;
import net.sf.l2j.gameserver.Geo.geoeditorcon.GeoEditorListener;
import net.sf.l2j.gameserver.datatables.xml.CustomMessages;
import net.sf.l2j.gameserver.datatables.xml.SystemMessages;
import net.sf.l2j.gameserver.handler.*;
import net.sf.l2j.gameserver.idfactory.IdFactory;
import net.sf.l2j.gameserver.instancemanager.*;
import net.sf.l2j.gameserver.instancemanager.grandbosses.VanHalterManager;
import net.sf.l2j.gameserver.instancemanager.lastimperialtomb.*;
import net.sf.l2j.gameserver.model.*;
import net.sf.l2j.gameserver.model.entity.Hero;
import net.sf.l2j.gameserver.network.*;
import net.sf.l2j.gameserver.script.faenor.FaenorScriptEngine;
import net.sf.l2j.gameserver.scripting.L2ScriptEngineManager;
import net.sf.l2j.gameserver.taskmanager.*;
import net.sf.l2j.gameserver.util.*;
import net.sf.l2j.status.Status;
import net.sf.l2j.util.Prints;
import net.sf.olympiad.Olympiad;
import org.mmocore.network.*;

import java.io.*;
import java.net.*;
import java.util.Calendar;
import java.util.logging.*;

/**
 *
 * @author Linkin
 * @update S-Project
 */
public class GameServer
{
	private static final Logger _log = Logger.getLogger(GameServer.class.getName());
	private final SelectorThread<L2GameClient> _selectorThread;
	private final SkillTable _skillTable;
	private final ItemTable _itemTable;
	private final NpcTable _npcTable;
	private final HennaTable _hennaTable;
	private final IdFactory _idFactory;
	public static GameServer gameServer;
	private static ClanHallManager _cHManager;
	private final ItemHandler _itemHandler;
	private final SkillHandler _skillHandler;
	private final AdminCommandHandler _adminCommandHandler;
	private final Shutdown _shutdownHandler;
	private final UserCommandHandler _userCommandHandler;
    private final VoicedCommandHandler _voicedCommandHandler;
    private final DoorTable _doorTable;
    private final SevenSigns _sevenSignsEngine;
    private final AutoChatHandler _autoChatHandler;
	private final AutoSpawnHandler _autoSpawnHandler;
	private LoginServerThread _loginThread;
    private final HelperBuffTable _helperBuffTable;

	private static Status _statusServer;
	@SuppressWarnings("unused")
	private final ThreadPoolManager _threadpools;

    public static final Calendar dateTimeServerStarted = Calendar.getInstance();

    public long getUsedMemoryMB()
	{
		return (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory())/1048576;
	}

    public SelectorThread<L2GameClient> getSelectorThread()
    {
    	return _selectorThread;
    }

	public ClanHallManager getCHManager()
    {
		return _cHManager;
	}
	public GameServer() throws Exception
	{
		_log.info("-----------------------------------------------------------------------------");
		_log.info("                                 S - Project                                 ");
		_log.info("-----------------------------------------------------------------------------");
        gameServer = this;
		_log.finest("Used memory: " + getUsedMemoryMB()+"mb" );
		_idFactory = IdFactory.getInstance();
        if (!_idFactory.isInitialized())
        {
            _log.severe("Could not read object IDs from DB. Please Check Your Data.");
            throw new Exception("Could not initialize the ID factory");
        }

		new File(Config.DATAPACK_ROOT, "data/clans").mkdirs();
		new File(Config.DATAPACK_ROOT, "data/crests").mkdirs();
		
		Prints.printSection("Cache");
		HtmCache.getInstance();
        CrestCache.getInstance();
		_threadpools = ThreadPoolManager.getInstance();
		
		Prints.printSection("Load Controllers");
		GameTimeController.getInstance();
		TradeController.getInstance();
		RecipeController.getInstance();
		
		Prints.printSection("Load Character");
		CharNameTable.getInstance();
		CharTemplateTable.getInstance();
		Heroes.getInstance().engineInit();
		Hero.getInstance();
		LevelUpData.getInstance();

		Prints.printSection("Load Tables");
		NpcWalkerRoutesTable.getInstance().load();
		GmListTable.getInstance();
        L2PetDataTable.getInstance().loadPetsData();
		ExtractableItemsData.getInstance();
		SummonItemsData.getInstance();
		TeleportLocationTable.getInstance();
		SkillTreeTable.getInstance();
		MapRegionTable.getInstance();
		ArmorSetsTable.getInstance();
		FishTable.getInstance();
		SkillSpellbookTable.getInstance();
		NobleSkillTable.getInstance();
		HeroSkillTable.getInstance();
        ClanTable.getInstance();
		_skillTable = SkillTable.getInstance();
		if (!_skillTable.isInitialized())
		{
		    _log.severe("Could not find the extraced files. Please Check Your Data.");
		    throw new Exception("Could not initialize the skill table");
		}
		
		_itemTable = ItemTable.getInstance();
		if (!_itemTable.isInitialized())
		{
		    _log.severe("Could not find the extraced files. Please Check Your Data.");
		    throw new Exception("Could not initialize the item table");
		}
		
		_npcTable = NpcTable.getInstance();
		if (!_npcTable.isInitialized())
		{
		    _log.severe("Could not find the extraced files. Please Check Your Data.");
		    throw new Exception("Could not initialize the npc table");
		}

		_hennaTable = HennaTable.getInstance();

		HennaTreeTable.getInstance();

        _helperBuffTable = HelperBuffTable.getInstance();

        if (!_helperBuffTable.isInitialized())
        {
           throw new Exception("Could not initialize the Helper Buff Table");
        }
    
		Prints.printSection("Load xml Messages");
		CustomMessages.getInstance();
		SystemMessages.getInstance();

		Prints.printSection("Load GeoData");
		GeoData.getInstance();
    
		Prints.printSection("Load L2World and L2Manor");
		L2World.getInstance();
		L2Manor.getInstance();
        
		Prints.printSection("Load Managers");
        _cHManager = ClanHallManager.getInstance();
		CastleManager.getInstance();
		SiegeManager.getInstance();
		CrownManager.getInstance();
		CursedWeaponsManager.getInstance();
		ZoneData.getInstance();
        SpawnTable.getInstance();
        RaidBossSpawnManager.getInstance();
        DayNightSpawnManager.getInstance().notifyChangeMode();
        DimensionalRiftManager.getInstance();
		Announcements.getInstance();
		TaskManager.getInstance();
		EventDroplist.getInstance();
		AuctionManager.getInstance();
		BoatManager.getInstance();
		KnownListUpdateTaskManager.getInstance();
		CastleManorManager.getInstance();
		MercTicketManager.getInstance();
		PetitionManager.getInstance();
        _doorTable = DoorTable.getInstance();
        _doorTable.parseData();
		GrandBossManager.getInstance();
		FourSepulchersManager.getInstance().init();
        if(!Config.Loaded_Frintezza)
        {
            Prints.printSection("Load Frintezza,VanHalter manager");
            VanHalterManager.getInstance().init();
		    LastImperialTombManager.getInstance().init();
		    FrintezzaManager.getInstance().init();
            LastImperialTombSpawnlist.getInstance();
        }
		if(Config.L2JMOD_ALLOW_WEDDING)
		CoupleManager.getInstance();
		QuestManager.getInstance();
		try
        {
            _log.info("Load Server Scripts");
            File scripts = new File("./Config/scripts.ini");
            L2ScriptEngineManager.getInstance().executeScriptList(scripts);
        }
        catch (IOException ioe)
        {
            _log.severe("Failed loading scripts.cfg, no script going to be loaded");
        }
        
        QuestManager.getInstance().report();
		AugmentationData.getInstance();
		if (Config.SAVE_DROPPED_ITEM)
			ItemsOnGroundManager.getInstance();

		if (Config.AUTODESTROY_ITEM_AFTER > 0 || Config.HERB_AUTO_DESTROY_TIME > 0)
    	    ItemsAutoDestroy.getInstance();

        if(!Config.ALT_DEV_NO_AI)
        {
            Prints.printSection("Load AI");
			AILoader.init();
        }
        //Загрузка Епик АИ
        if(!Config.Sailren_Load)
        {
             SailrenLoader.init();
        }
        if(!Config.Core_Load)
        {
             CoreLoader.init();
        }
        if(!Config.Zaken_Load)
        {
             ZakenLoader.init();
        }
        if(!Config.Orfen_Load)
        {
             OrfenLoader.init();
        }
        if(!Config.QueentAnt_Load)
        {
             QueenAntLoader.init();
        }
        if(!Config.Baium_Load)
        {
             BaiumLoader.init();
        }
        if(!Config.Antharas_Load)
        {
             AntharasLoader.init();
        }
        if(!Config.Valakas_Load)
        {
             ValakasLoader.init();
        }
        if(!Config.Frintezza_Load)
        {
             FrintezzaLoader.init();
        }
        if(!Config.VanHalter_Load)
        {
             VanHalterLoader.init();
        }
        Prints.printSection("Load Seven Signs");
		_sevenSignsEngine = SevenSigns.getInstance();
        SevenSignsFestival.getInstance();
		_autoSpawnHandler = AutoSpawnHandler.getInstance();
		_autoChatHandler = AutoChatHandler.getInstance();
        _sevenSignsEngine.spawnSevenSignsNPC();

        Prints.printSection("Load Olympiad");
        Olympiad.getInstance();
   
        Prints.printSection("Load Other");
        FaenorScriptEngine.getInstance();
        MonsterRace.getInstance();
        StaticObjects.getInstance();

		_log.config("*** AutoChat: " + _autoChatHandler.size() + " loaded in total.");
		_log.config("*** AutoSpawn: " + _autoSpawnHandler.size() + " loaded in total.");

		_itemHandler = ItemHandler.getInstance();
        _log.config("*** Items: " + _itemHandler.size() + " loaded.");

		_skillHandler = SkillHandler.getInstance();
        _log.config("*** Skills: " + _skillHandler.size() + " loaded.");

		_adminCommandHandler = AdminCommandHandler.getInstance();
        _log.config("*** AdminCommand: " + _adminCommandHandler.size() + " loaded.");

        _userCommandHandler = UserCommandHandler.getInstance();
        _log.config("*** UserCommand: " + _userCommandHandler.size() + " loaded.");

		_voicedCommandHandler = VoicedCommandHandler.getInstance();
		_log.config("*** VoicedCommand: " + _voicedCommandHandler.size() + " loaded.");
		
        Universe.getInstance();

		GeoEditorListener.getInstance();

		_shutdownHandler = Shutdown.getInstance();
		Runtime.getRuntime().addShutdownHook(_shutdownHandler);

		try
        {
            _doorTable.getDoor(24190001).openMe();
            _doorTable.getDoor(24190002).openMe();
            _doorTable.getDoor(24190003).openMe();
            _doorTable.getDoor(24190004).openMe();
            _doorTable.getDoor(23180001).openMe();
            _doorTable.getDoor(23180002).openMe();
            _doorTable.getDoor(23180003).openMe();
            _doorTable.getDoor(23180004).openMe();
            _doorTable.getDoor(23180005).openMe();
            _doorTable.getDoor(23180006).openMe();
            _doorTable.checkAutoOpen();
        }
        catch (NullPointerException e)
        {
            _log.warning("There is errors in your Door.csv file. Update door.csv");
        }
        ForumsBBSManager.getInstance();
        _log.config("IdFactory: Free ObjectID's remaining: " + IdFactory.getInstance().size());
        try
        {
            DynamicExtension.getInstance();
        }
        catch (Exception ex)
        {
            _log.log(Level.WARNING, "DynamicExtension could not be loaded and initialized", ex);
        }
        PowerPack.getInstance();
        Prints.printSection("Load FloodProtected");
        FloodProtector.getInstance();
        Prints.printSection("Load Event");
        EventManager.getInstance().startEventRegistration();
		if(Config.pccafe_event) PcCafe.getInstance();
        Prints.printSection("Load Buffer");
        CharShemesTable.getInstance();
        Prints.printSection("Load Other-2");
        if ((Config.OFFLINE_TRADE_ENABLE || Config.OFFLINE_CRAFT_ENABLE) && Config.RESTORE_OFFLINERS)
			OfflineTradeTable.restoreOfflineTraders(); 
		System.gc();
		long freeMem = (Runtime.getRuntime().maxMemory()-Runtime.getRuntime().totalMemory()+Runtime.getRuntime().freeMemory()) / 1048576;
		long totalMem = Runtime.getRuntime().maxMemory() / 1048576;
		_log.info("========================================================================");
		_log.info("Game Server started, free memory "+freeMem+" Mb of "+totalMem+" Mb");

		_loginThread = LoginServerThread.getInstance();
		_loginThread.start();
        
        L2GamePacketHandler gph = new L2GamePacketHandler();
		SelectorConfig<L2GameClient> sc = new SelectorConfig<L2GameClient>(null, null, gph, gph);
        sc.setMaxSendPerPass(12);
        sc.setSelectorSleepTime(20);

		_selectorThread = new SelectorThread<L2GameClient>(sc, gph, gph, null);
		InetAddress bindAddress = null;
		if (!Config.GAMESERVER_HOSTNAME.equals("*"))
		{
			try
			{
				bindAddress = InetAddress.getByName(Config.GAMESERVER_HOSTNAME);
			}
			catch (UnknownHostException e1)
			{
				_log.severe("WARNING: The GameServer bind address is invalid, using all avaliable IPs. Reason: "+e1.getMessage());
				if (Config.DEVELOPER)
				{
					e1.printStackTrace();
				}
			}
		}
		
		try
		{
			_selectorThread.openServerSocket(bindAddress, Config.PORT_GAME);
		}
		catch (IOException e)
		{
			_log.severe("FATAL: Failed to open server socket. Reason: "+e.getMessage());
			if (Config.DEVELOPER)
			{
				e.printStackTrace();
			}
			System.exit(1);
		}
		_selectorThread.start();
		_log.config("Maximum Numbers of Connected Players: " + Config.MAXIMUM_ONLINE_USERS);
        int restartAt;
		if(Config.AutoRestartAt > -1)
		{
			final Calendar calendarRestartAt = Calendar.getInstance();
			calendarRestartAt.set(Calendar.HOUR_OF_DAY, Config.AutoRestartAt);
			calendarRestartAt.set(Calendar.MINUTE, 0);

			if(calendarRestartAt.getTimeInMillis() < System.currentTimeMillis())
				calendarRestartAt.add(Calendar.HOUR, 24);
			restartAt = (int) (calendarRestartAt.getTimeInMillis() - System.currentTimeMillis()) / 1000;
			Shutdown.getInstance().setAutoRestart(restartAt);
		}
	}

	public static void main(String[] args) throws Exception
    {
		if(System.getProperty("os.name").startsWith("Windows"))
		{
			//Вывод русских символов на консоль
			PrintStream ps= new PrintStream(System.out,true,"Cp866");
			System.setOut(ps);
			System.setErr(ps);
		}
		Server.serverMode = Server.MODE_GAMESERVER;
		final String LOG_FOLDER = "Logs";
		final String LOG_NAME = "./Config/log.ini";
		File logFolder = new File(Config.DATAPACK_ROOT, LOG_FOLDER);
		logFolder.mkdir();
		InputStream is =  new FileInputStream(new File(LOG_NAME));
		LogManager.getLogManager().readConfiguration(is);
		is.close();
		Config.load();
		L2DatabaseFactory.getInstance();
		gameServer = new GameServer();

		if ( Config.IS_TELNET_ENABLED)
        {
		    _statusServer = new Status(Server.serverMode);
		    _statusServer.start();
		}
		else
        {
		    System.out.println("Telnet server is currently disabled.");
		}
    }
}