/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver.ai.special;

import net.sf.l2j.gameserver.ai.CtrlIntention;
import net.sf.l2j.gameserver.ai.L2AttackableAIScript;
import net.sf.l2j.gameserver.datatables.xml.CustomMessages;
import net.sf.l2j.gameserver.model.*;
import net.sf.l2j.gameserver.model.actor.instance.*;
import net.sf.l2j.gameserver.model.quest.Quest;
import net.sf.l2j.gameserver.network.serverpackets.CreatureSay;
import net.sf.l2j.gameserver.util.Util;
import net.sf.l2j.util.Rnd;

import javolution.util.FastList;
import javolution.util.FastMap;

/**
 * 
 * @author S-Project
 * @update S-Project
 */
public class Monastery extends L2AttackableAIScript
{
	public Monastery(int questId, String name, String descr)
	{
		super(questId, name, descr);
		int[] mobs = {22124, 22125, 22126, 22127, 22129};
		this.registerMobs(mobs);
	}

	private FastMap<Integer, FastList<L2Character>> _attackersList = new FastMap<Integer, FastList<L2Character>>();

	private static boolean _isAttacked = false;

	public String onAttack(L2NpcInstance npc, L2PcInstance attacker, int damage, boolean isPet, L2Skill skill)
	{
		int npcObjId = npc.getObjectId();

		L2Character target = isPet ? attacker.getPet() : attacker;

		if (npc.getNpcId() == 22129 && !isPet && !_isAttacked && Rnd.get(100) < 50 && attacker.getActiveWeaponItem() != null)
        {
            String message = CustomMessages.getInstance().getMessage(3);
            message = message.replaceAll("%username%", attacker.getName());
            npc.broadcastPacket(new CreatureSay(npc.getObjectId(), 0, npc.getName(), message));
        }

		if (_attackersList.get(npcObjId) == null)
		{
			FastList<L2Character> player = new FastList<L2Character>();
			player.add(target);
			_attackersList.put(npcObjId, player);
		}
		else if (!_attackersList.get(npcObjId).contains(target))
			_attackersList.get(npcObjId).add(target);

		_isAttacked = true;

		return super.onAttack(npc, attacker, damage, isPet);
	}

	public String onAggroRangeEnter(L2NpcInstance npc, L2PcInstance player, boolean isPet)
	{
		int npcObjId = npc.getObjectId();

		L2Character target = isPet ? player.getPet() : player;

		if (player.getActiveWeaponItem() != null)
		{
			if (npc.getNpcId() == 22129)
			{
                String message = CustomMessages.getInstance().getMessage(3);
                message = message.replaceAll("%username%", target.getName());
                npc.broadcastPacket(new CreatureSay(npc.getObjectId(), 0, npc.getName(), message));
            }
			else
            {
                String message = CustomMessages.getInstance().getMessage(Rnd.get(1,2));
                message = message.replaceAll("%username%", target.getName());
                npc.broadcastPacket(new CreatureSay(npc.getObjectId(), 0, npc.getName(), message));
            }
			((L2Attackable) npc).addDamageHate(target, 0, 999);
			npc.getAI().setIntention(CtrlIntention.AI_INTENTION_ATTACK, target);
		}
		else
		{
			if (_attackersList.get(npcObjId) == null || !_attackersList.get(npcObjId).contains(target))
				((L2Attackable) npc).getAggroListRP().remove(target);
			else
			{
				((L2Attackable) npc).addDamageHate(target, 0, 999);
				npc.getAI().setIntention(CtrlIntention.AI_INTENTION_ATTACK, target);
			}
		}

		return super.onAggroRangeEnter(npc, player, isPet);
	}

	public String onKill(L2NpcInstance npc, L2PcInstance killer, boolean isPet)
	{
		int npcObjId = npc.getObjectId();
		_isAttacked = false;
		if (_attackersList.get(npcObjId) != null)
			_attackersList.get(npcObjId).clear();

		return super.onKill(npc, killer, isPet);
	}

	public static void main(String[] args)
	{
		new Monastery(-1, "Monastery", "ai");
	}
}