/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver.datatables;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import javolution.util.FastList;
import javolution.util.FastMap;

import net.sf.l2j.Config;
import net.sf.l2j.gameserver.model.L2HennaInstance;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


import net.sf.l2j.gameserver.model.base.ClassId;
import net.sf.l2j.gameserver.templates.L2Henna;

/**
 * Create by Incognito
 */

public class HennaTreeTable
{
	private static Logger _log = Logger.getLogger(HennaTreeTable.class.getName());
	private static final HennaTreeTable _instance = new HennaTreeTable();
	private Map<ClassId, List<L2HennaInstance>> _hennaTrees;

	public static HennaTreeTable getInstance()
	{
		return _instance;
	}

	private HennaTreeTable()
	{
		_hennaTrees = new FastMap<ClassId, List<L2HennaInstance>>();
		int classId = 0;
		int count = 0;
		List<L2HennaInstance> list = new FastList<L2HennaInstance>();
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		factory.setIgnoringComments(true);
		File f = new File(Config.DATAPACK_ROOT, "/data/custom/xml/Henna/henna_tree.xml");
		if(!f.exists())
		{
			_log.warning("henna_tree.xml could not be loaded: file not found");
			return;
		}
		try
		{
			InputSource in = new InputSource(new InputStreamReader(new FileInputStream(f), "UTF-8"));
			in.setEncoding("UTF-8");
			Document doc = factory.newDocumentBuilder().parse(in);
			for(Node n = doc.getFirstChild(); n != null; n = n.getNextSibling())
			{
				if(n.getNodeName().equalsIgnoreCase("list"))
				{
					for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
					{
						if(d.getNodeName().equalsIgnoreCase("henna"))
						{
							classId = Integer.valueOf(d.getAttributes().getNamedItem("class_id").getNodeValue());
							for(Node t = d.getFirstChild(); t != null; t = t.getNextSibling())
							{
								if(t.getNodeName().equalsIgnoreCase("symbol"))
								{
									int id = Integer.valueOf(t.getAttributes().getNamedItem("id").getNodeValue());
									L2Henna template = HennaTable.getInstance().getTemplate(id);
									if(template != null)
									{
										L2HennaInstance temp = new L2HennaInstance(template);
										temp.setSymbolId(id);
										temp.setItemIdDye(template.getDyeId());
										temp.setAmountDyeRequire(template.getAmountDyeRequire());
										temp.setPrice(template.getPrice());
										temp.setStatINT(template.getStatINT());
										temp.setStatSTR(template.getStatSTR());
										temp.setStatCON(template.getStatCON());
										temp.setStatMEM(template.getStatMEM());
										temp.setStatDEX(template.getStatDEX());
										temp.setStatWIT(template.getStatWIT());

										list.add(temp);
										temp = null;
										template = null;
									}
								}
							}
						}
					}
				}
			}
		}
		catch(SAXException e)
		{
			_log.warning("Error while creating table: " + e.getMessage() + "\n" + e);
		}
		catch(IOException e)
		{
			_log.warning("Error while creating table: " + e.getMessage() + "\n" + e);
		}
		catch(ParserConfigurationException e)
		{
			_log.warning("Error while creating table: " + e.getMessage() + "\n" + e);
		}

		_hennaTrees.put(ClassId.values()[classId], list);
		count += list.size();
		list = null;

		_log.config("HennaTreeTable: Loaded " + count + " Henna Tree Templates.");
	}

	public L2HennaInstance[] getAvailableHenna(ClassId classId)
	{
		List<L2HennaInstance> result = new FastList<L2HennaInstance>();
		List<L2HennaInstance> henna = _hennaTrees.get(classId);
		if(henna == null)
		{
			// the hennatree for this class is undefined, so we give an empty list
			_log.warning("Hennatree for class " + classId + " is not defined !");

			return new L2HennaInstance[0];
		}

		for(int i = 0; i < henna.size(); i++)
		{
			L2HennaInstance temp = henna.get(i);
			result.add(temp);
			temp = null;
		}
		henna = null;

		return result.toArray(new L2HennaInstance[result.size()]);
	}
}