/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

/**
 *
 * @author FBIagent
 *
 */
package net.sf.l2j.gameserver.datatables;

import javolution.util.FastMap;
import net.sf.l2j.Config;
import net.sf.l2j.gameserver.model.L2SummonItem;

import java.io.File;
import java.util.Scanner;
import org.w3c.dom.*;
import org.xml.sax.*;
import java.io.*;
import java.util.logging.Logger;
import javax.xml.parsers.*;

import javolution.util.FastMap;

/**
 * @update S-Project
 */
public class SummonItemsData
{
    private static Logger _log = Logger.getLogger(SummonItemsData.class.getName());
	private FastMap<Integer, L2SummonItem> _summonitems;

	private static SummonItemsData _instance;

	public static SummonItemsData getInstance()
	{
		if (_instance == null)
			_instance = new SummonItemsData();

		return _instance;
	}

	public SummonItemsData()
	{
		_summonitems = new FastMap<Integer, L2SummonItem>();
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		factory.setIgnoringComments(true);
        File f = new File(Config.DATAPACK_ROOT, "/data/custom/xml/summon_items.xml");
		if(!f.exists())
		{
			_log.warning("summon_items.xml could not be loaded: file not found");
			return;
		}
		int itemID = 0, npcID = 0;
		byte summonType = 0;
		try
		{
			InputSource in = new InputSource(new InputStreamReader(new FileInputStream(f), "UTF-8"));
			in.setEncoding("UTF-8");
			Document doc = factory.newDocumentBuilder().parse(in);
			for(Node n = doc.getFirstChild(); n != null; n = n.getNextSibling())
			{
				if(n.getNodeName().equalsIgnoreCase("list"))
				{
					for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
					{
						if(d.getNodeName().equalsIgnoreCase("summon_item"))
						{
							itemID = Integer.valueOf(d.getAttributes().getNamedItem("itemID").getNodeValue());
							npcID = Integer.valueOf(d.getAttributes().getNamedItem("npcID").getNodeValue());
							summonType = Byte.valueOf(d.getAttributes().getNamedItem("summonType").getNodeValue());
							L2SummonItem summonitem = new L2SummonItem(itemID, npcID, summonType);
							_summonitems.put(itemID, summonitem);
						}
					}
				}
			}
		}
		catch(SAXException e)
		{
			_log.warning("Error while creating table: " + e.getMessage() + "\n" + e);
		}
		catch(IOException e)
		{
			_log.warning("Error while creating table: " + e.getMessage() + "\n" + e);
		}
		catch(ParserConfigurationException e)
		{
			_log.warning("Error while creating table: " + e.getMessage() + "\n" + e);
		}

		System.out.println("Summon items data: Loaded " + _summonitems.size() + " summon items.");
	}

	public L2SummonItem getSummonItem(int itemId)
	{
		return _summonitems.get(itemId);
	}
	public int[] itemIDs()
	{
		int size = _summonitems.size();
		int[] result = new int[size];
		int i = 0;
		for (L2SummonItem si : _summonitems.values())
        {
			result[i] = si.getItemId();
			i++;
        }
		return result;
	}
}