/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver.datatables.xml;

import java.io.*;
import java.util.*;

import javax.xml.parsers.*;
import net.sf.l2j.Config;
import org.apache.commons.logging.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * @update S-Project
 */
public final class CustomMessages
{
	private final static Log _log = LogFactory.getLog(CustomMessages.class);

	public static CustomMessages getInstance()
	{
		return instance._instance;
	}

	private Map<Integer, String> _messages;

	public CustomMessages()
	{
		System.out.print("Loading custom messages...");
		_messages = new HashMap<Integer, String>();
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		factory.setIgnoringComments(true);
		File f = new File(Config.DATAPACK_ROOT, "/data/custom/xml/Messages/custom_message.xml");
		if(!f.exists())
		{
			_log.error("custom_message.xml could not be loaded: file not found");
			return;
		}
		try
		{
			InputSource in = new InputSource(new InputStreamReader(new FileInputStream(f), "UTF-8"));
			in.setEncoding("UTF-8");
			Document doc = factory.newDocumentBuilder().parse(in);
			for(Node n = doc.getFirstChild(); n != null; n = n.getNextSibling())
			{
				if(n.getNodeName().equalsIgnoreCase("list"))
				{
					for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
					{
						if(d.getNodeName().equalsIgnoreCase("message"))
						{
							_messages.put(Integer.valueOf(d.getAttributes().getNamedItem("id").getNodeValue()), String.valueOf(d.getFirstChild().getNodeValue()));
						}
					}
				}
			}
		}
		catch(SAXException e)
		{
			_log.error("", e);
		}
		catch(IOException e)
		{
			_log.error("", e);
		}
		catch(ParserConfigurationException e)
		{
			_log.error("", e);
		}

		System.out.println("\t" + _messages.size());
	}

	public String getMessage(int id)
	{
		if(_messages.get(id)!=null)
			return _messages.get(id);
		else
			return String.valueOf(id);
	}

	static class instance
	{
		static final CustomMessages _instance = new CustomMessages();
	}
}