/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver.datatables.xml;

import java.io.*;
import java.util.*;

import javax.xml.parsers.*;

import javolution.util.FastList;

import org.apache.commons.logging.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import net.sf.l2j.Config;

/**
 * @update S-Project
 */
public final class SystemMessages
{
	private final static Log _log = LogFactory.getLog(SystemMessages.class);
	public static SystemMessages getInstance()
	{
		return instance._instance;
	}

	private final Map<Integer, FastList<String>> _message;

	/**
	 * -1 - client<br>
	 * 0 - localize 0<br>
	 * 1 - localize 1<br>
	 * ...
	 */
	private byte _type = -1;

	public SystemMessages()
	{
		System.out.print("Loading system messages...");
		_message = new HashMap<Integer, FastList<String>>();
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		factory.setIgnoringComments(true);
		File f = new File(Config.DATAPACK_ROOT, "/data/custom/xml/Messages/system_message.xml");
		if(!f.exists())
		{
			_log.error("system_message.xml could not be loaded: file not found");
			return;
		}
		try
		{
			InputSource in = new InputSource(new InputStreamReader(new FileInputStream(f), "UTF-8"));
			in.setEncoding("UTF-8");
			Document doc = factory.newDocumentBuilder().parse(in);
			int id = 0;
			FastList<String> messages = new FastList<String>();
			for(Node n = doc.getFirstChild(); n != null; n = n.getNextSibling())
			{
				if(n.getNodeName().equalsIgnoreCase("list"))
				{
					_type = Byte.valueOf(n.getAttributes().getNamedItem("type").getNodeValue());
					for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
					{
						if(!messages.isEmpty() && id > 0)
							_message.put(id, messages);
						messages.clear();
						id = 0;
						if(d.getNodeName().equalsIgnoreCase("message"))
						{
							id = Integer.valueOf(d.getAttributes().getNamedItem("id").getNodeValue());
							for(Node t = d.getFirstChild(); t != null; t = t.getNextSibling())
								if(t.getNodeName().equalsIgnoreCase("localize"))
									messages.add(t.getNodeValue());
						}
					}
				}
			}
		}
		catch(SAXException e)
		{
			_log.error("", e);
		}
		catch(IOException e)
		{
			_log.error("", e);
		}
		catch(ParserConfigurationException e)
		{
			_log.error("", e);
		}

		System.out.println("     "+ _message.size());
	}

	/**
	 *
	 * @param id
	 * @return
	 */
	public String getMessage(int id)
	{
		if(_type == -1 || _message.isEmpty())
			return String.valueOf(id);

		String s;
		for(byte b = _type; b < 0; b--)
		{
			s = _message.get(id).get(_type);
			if(s != null)
				return s;
		}
		return String.valueOf(id);
	}

	static class instance
	{
		final static SystemMessages _instance = new SystemMessages();
	}
}