package net.sf.l2j.gameserver.handler;

import javolution.util.FastMap;
import net.sf.l2j.Config;
import net.sf.l2j.gameserver.handler.voicedcommandhandlers.*;

import java.util.Map;
import java.util.logging.Logger;

/**
 *
 * @author TheFosT
 */
public class VoicedCommandHandler
{
	private static Logger _log = Logger.getLogger(ItemHandler.class.getName());

	private static VoicedCommandHandler _instance;

	private Map<String, IVoicedCommandHandler> _datatable;

	public static VoicedCommandHandler getInstance()
	{
		if (_instance == null)
		{
			_instance = new VoicedCommandHandler();
		}
		return _instance;
	}

	private VoicedCommandHandler()
	{
		_datatable = new FastMap<String, IVoicedCommandHandler>();
            registerVoicedCommandHandler(new stats());
        if(Config.ALLOW_IpBlock)
		    registerVoicedCommandHandler(new ipblock());
        if(Config.ALLOW_EXP_GAIN_COMMAND)
		    registerVoicedCommandHandler(new ExpGain());
		if(Config.ALLOW_ONLINE)
		    registerVoicedCommandHandler(new Online());
		if(Config.L2JMOD_ALLOW_WEDDING)
		    registerVoicedCommandHandler(new Wedding());
		if(Config.BANKING_SYSTEM_ENABLED)
			registerVoicedCommandHandler(new Banking());
		if(Config.CTF_COMMAND)
			registerVoicedCommandHandler(new CTFCmd());
		if(Config.TVT_COMMAND)
			registerVoicedCommandHandler(new TvTCmd());
		if(Config.DM_COMMAND)
			registerVoicedCommandHandler(new DMCmd());
        if (Config.ALLOW_TRADEOFF_COMMAND)
        	registerVoicedCommandHandler(new tradeoff());
        if (Config.ALLOW_PLAYERS_REFUSAL)
        	registerVoicedCommandHandler(new pmoff());
	}

	public void registerVoicedCommandHandler(IVoicedCommandHandler handler)
	{
		String[] ids = handler.getVoicedCommandList();
		for (int i = 0; i < ids.length; i++)
		{
			if (Config.DEBUG) _log.fine("Adding handler for command "+ids[i]);
			_datatable.put(ids[i], handler);
		}
	}

	public IVoicedCommandHandler getVoicedCommandHandler(String voicedCommand)
	{
		String command = voicedCommand;
		if (voicedCommand.indexOf(" ") != -1)
        {
			command = voicedCommand.substring(0, voicedCommand.indexOf(" "));
		}
		if (Config.DEBUG)
			_log.fine("getting handler for command: "+command+
					" -> "+(_datatable.get(command) != null));
		return _datatable.get(command);
	}

    public int size()
    {
        return _datatable.size();
    }
}