package net.sf.l2j.gameserver.handler.itemhandlers;

import net.sf.l2j.gameserver.datatables.DoorTable;
import net.sf.l2j.gameserver.handler.IItemHandler;
import net.sf.l2j.gameserver.model.*;
import net.sf.l2j.gameserver.model.actor.instance.*;
import net.sf.l2j.gameserver.network.SystemMessageId;
import net.sf.l2j.gameserver.network.serverpackets.ActionFailed;

/**
 * @update S-Project
 */
public class Key implements IItemHandler
{
        private int ITEM_IDS[] =
            {
                    8056
            };
        public static final int INTERACTION_DISTANCE = 100;

        public void useItem(L2PlayableInstance playable, L2ItemInstance item)
        {
                if (!(playable instanceof L2PcInstance))
                        return;
                L2PcInstance activeChar = (L2PcInstance) playable;
                L2Object target = activeChar.getTarget();

                if (target == null || !(target instanceof L2DoorInstance))
                {
                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                        activeChar.sendPacket(new ActionFailed());
                        return;
                }
                L2DoorInstance door = (L2DoorInstance) target;

                if (door.getOpen())
                {
                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                        activeChar.sendPacket(new ActionFailed());
                        return;
                }

                if (!(activeChar.isInsideRadius(door, INTERACTION_DISTANCE, false, false)))
                {
                        activeChar.sendMessage("Too far.");
                        activeChar.sendPacket(new ActionFailed());
                        return;
                }
                if (activeChar.getAbnormalEffect() > 0 || activeChar.isInCombat())
                {
                        activeChar.sendMessage("You are currently enganged in combat.");
                        activeChar.sendPacket(new ActionFailed());
                        return;
                }

                if (!playable.destroyItem("Consume", item.getObjectId(), 1, null, false))
                        return;

                switch (item.getItemId())
                {
                        case 8056:
                                if (door.getDoorId() == 23150004 || door.getDoorId() == 23150003)
                                {
                                        DoorTable.getInstance().getDoor(23150003).openMe();
                                        DoorTable.getInstance().getDoor(23150003).onOpen();
                                        DoorTable.getInstance().getDoor(23150004).openMe();
                                        DoorTable.getInstance().getDoor(23150004).onOpen();
                                }
                                else
                                {
                                        activeChar.sendMessage("Incorrect Door.");
                                }
                                break;
                        case 9685: // Gate Key: Darkness
                                if (door.getDoorId() != 20260001)
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9686: // Gate Key: Destruction
                                if (door.getDoorId() != 20260006)
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9687: // Gate Key: Blood
                                if ((door.getDoorId() != 20260002) && (door.getDoorId() != 20260005))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9694: // Secret Key
                                if ((door.getDoorId() != 24220001)
                                                && (door.getDoorId() != 24220002)
                                                && (door.getDoorId() != 24220003)
                                                && (door.getDoorId() != 24220004))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9704: // Gate Key: Archives
                                if ((door.getDoorId() != 16200005))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9705: // Gate Key: Observation
                                if ((door.getDoorId() != 16200009))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9706: // Gate Key: Spicula
                                if ((door.getDoorId() != 16200003))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9707: // Gate Key: Harkilgamed
                                if ((door.getDoorId() != 16200007))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9708: // Gate Key: Rodenpicula
                                if ((door.getDoorId() != 16200008))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9709: // Gate Key: Arviterre
                                if ((door.getDoorId() != 16200010))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9710: // Gate Key: Katenar
                                if ((door.getDoorId() != 16200006))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9711: // Gate Key: Prediction
                                if ((door.getDoorId() != 16200011))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 9712: // Gate Key: Massive Cavern
                                if ((door.getDoorId() != 16200012))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                        case 10015: // Prison Gate Key
                                if ((door.getDoorId() != 24220008))
                                {
                                        activeChar.sendPacket(SystemMessageId.INCORRECT_TARGET);
                                        return;
                                }
                                door.openMe();
                                break;
                }
}
        public int[] getItemIds()
        {
                return ITEM_IDS;
        }
}