package net.sf.l2j.gameserver.handler.skillhandlers;

import net.sf.l2j.gameserver.ThreadPoolManager;
import net.sf.l2j.gameserver.handler.ISkillHandler;
import net.sf.l2j.gameserver.instancemanager.CastleManager;
import net.sf.l2j.gameserver.instancemanager.GrandBossManager;
import net.sf.l2j.gameserver.model.*;
import net.sf.l2j.gameserver.model.actor.instance.L2PcInstance;
import net.sf.l2j.gameserver.model.entity.Castle;
import net.sf.l2j.gameserver.network.SystemMessageId;
import net.sf.l2j.gameserver.network.serverpackets.SystemMessage;
import net.sf.l2j.gameserver.model.L2Skill.SkillType;

/**
 * @update S-Project
 */
public class ClanGate implements ISkillHandler
{
	private static final SkillType[] CG_SKILLS =
            {
                    SkillType.CLAN_GATE
            };

	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.handler.ISkillHandler#getSkillIds()
	 */
	@Override
	public SkillType[] getSkillIds()
    {
		return CG_SKILLS;
	}

	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.handler.ISkillHandler#useSkill(net.sf.l2j.gameserver.model.L2Character, net.sf.l2j.gameserver.model.L2Skill, net.sf.l2j.gameserver.model.L2Character[])
	 */
	@Override
	//public void useSkill(L2Character activeChar, L2Skill skill, L2Character... targets)
    public void useSkill(L2Character activeChar, L2Skill skill, L2Object[] targets)
    {
		L2PcInstance player = null;
		if (activeChar instanceof L2PcInstance)
			player = (L2PcInstance) activeChar;
		else
			return;

		//need more checking...
          if(player.isInsideZone(L2Character.ZONE_NOLANDING)
                     || player.isInOlympiadMode() || player.isInsideZone(L2Character.ZONE_PVP)
                     || GrandBossManager.getInstance().getZone(player) != null)
             {
                     player.sendMessage("Нельзя открыть здесь портал.");
                     return;
             }

		L2Clan clan = player.getClan();
		if (clan != null)
        {
			if(CastleManager.getInstance().getCastleByOwner(clan) != null)
            {
				Castle castle = CastleManager.getInstance().getCastleByOwner(clan);
				if (player.isCastleLord(castle.getCastleId()))
                {
					//please note clan gate expires in two minutes WHATEVER happens to the clan leader.
					ThreadPoolManager.getInstance().scheduleGeneral(new RemoveClanGate(castle.getCastleId()), skill.getTotalLifeTime());
					castle.createClanGate(activeChar.getX(), activeChar.getY(), activeChar.getZ() + 20);
					player.getClan().broadcastToOnlineMembers(new SystemMessage(SystemMessageId.THE_PORTAL_HAS_BEEN_CREATED));
				}
			}
		}
		L2Effect effect = player.getFirstEffect(skill.getId());
		if (effect != null && effect.isSelfEffect())
			effect.exit();
		skill.getEffectsSelf(player);
	}

	private class RemoveClanGate implements Runnable
    {
		private final int castle;

        private RemoveClanGate(int castle)
        {
        	this.castle = castle;
        }

        /* (non-Javadoc)
         * @see java.lang.Runnable#run()
         */
        @Override
        public void run()
        {
        	CastleManager.getInstance().getCastleById(castle).destroyClanGate();
        }
    }
}