package net.sf.l2j.gameserver.model.actor.instance;

import javolution.util.FastList;
import javolution.util.FastMap;
import net.sf.PowerPack.PowerPackConfig;
import net.sf.l2j.Config;
import net.sf.l2j.gameserver.datatables.*;
import net.sf.l2j.gameserver.model.*;
import net.sf.l2j.gameserver.network.serverpackets.ActionFailed;
import net.sf.l2j.gameserver.network.serverpackets.NpcHtmlMessage;
import net.sf.l2j.gameserver.taskmanager.AttackStanceTaskManager;
import net.sf.l2j.gameserver.templates.L2NpcTemplate;
import net.sf.olympiad.Olympiad;
//load event
import net.sf.Event.CTF;
import net.sf.Event.TvT;
import net.sf.Event.DM;

import java.util.StringTokenizer;

/**
 * @author TheFosT
 * @update S-Project
 */
public class L2BuffInstance extends L2FolkInstance
{
    public int[] TableId;
    public int[] TableDialog;

    public L2BuffInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
        TableId=new int[]
          {
                1068,1388,1086,1077,1242,1240,1085,1059,1303,1062,1043,
                1040,1389,1036,1035,1243,1304,1078,1087,
                1045,1048,1204,1073,1397,1044,1268,1257,
                1182,1189,1191,1033,1259,1392,1393,1353,1352,1354,
                271,272,273,274,275,276,277,307,309,310,311,
                264,265,266,267,268,269,270,304,305,306,308,
                1355,1356,1357,1363,4699,4554,365,363,364,349,4702,  //=70 buff
                4553,4552,4551,4703,4700
		  };
        TableDialog=new int[]
          {
                1,1,1,1,1,1,1,1,1,1,1,
				2,2,2,2,2,2,2,2,
				3,3,3,3,3,3,3,3,
				4,4,4,4,4,4,4,4,4,4,
				5,5,5,5,5,5,5,5,5,5,5,
				6,6,6,6,6,6,6,6,6,6,6,
				7,7,7,7,7,7,7,7,7,7,7,
                7,7,7
		   };
    }

    private boolean checkAllowed(L2PcInstance activeChar)
	{
		String msg = null;
		if (activeChar.isSitting())
			msg = "Бафер не доступен когда вы сидите";
		else if (activeChar.isCastingNow())
			msg = "Бафер не доступен когда вы используете заклинания";
		else if(Config.BUFFER_PROTECTION.contains("ALL"))
			msg = "Бафер не доступен в данной зоне";
		else if(Config.BUFFER_PROTECTION.contains("CURSED") && activeChar.isCursedWeaponEquiped())
			msg = "Бафер не доступен с проклятым оружием";
		else if(Config.BUFFER_PROTECTION.contains("ATTACK") && AttackStanceTaskManager.getInstance().getAttackStanceTask(activeChar))
			msg = "Бафер не доступен во время боя";
		else if(Config.BUFFER_PROTECTION.contains("DUNGEON") && activeChar.isIn7sDungeon())
			msg = "Бафер не доступен в катакомбах и некрополисах";
		else if(Config.BUFFER_PROTECTION.contains("RB") && activeChar.isInsideZone(L2Character.ZONE_NOSUMMONFRIEND))
			msg = "Бафер не доступен в данной зоне";
		else if(Config.BUFFER_PROTECTION.contains("PVP") && activeChar.isInsideZone(L2Character.ZONE_PVP))
			msg = "Бафер не доступен в данной зоне";
		else if(Config.BUFFER_PROTECTION.contains("PEACE") && activeChar.isInsideZone(L2Character.ZONE_PEACE))
			msg = "Бафер не доступен в данной зоне";
		else if(Config.BUFFER_PROTECTION.contains("SIEGE") && activeChar.isInsideZone(L2Character.ZONE_SIEGE))
			msg = "Бафер не доступен в данной зоне";
		else if(Config.BUFFER_PROTECTION.contains("OLYMPIAD") && (activeChar.isInOlympiadMode() ||
				activeChar.isInsideZone(L2Character.ZONE_OLY) || Olympiad.getInstance().isRegistered(activeChar) ||
				Olympiad.getInstance().isRegisteredInComp(activeChar)))
			msg = "Бафер не доступен на Великой Олимпиаде";
            else if(PowerPackConfig.BUFFER_EXCLUDE_ON.contains("EVENT") && activeChar.isInEvent())
               msg = "Бафер не доступен на эвенте";
		else if(activeChar.isDead())
			msg = "Бафер не доступен когда вы мертвы";
		else if (Config.BUFFER_PROTECTION.contains("EVENT") && (activeChar.isInFunEvent()))
            msg = "Buffer is not available in this event";
        else if (Config.BUFFER_PROTECTION.contains("TVT") && activeChar._inEventTvT && TvT.is_started())
            msg = "Buffer is not available in TVT";
        else if (Config.BUFFER_PROTECTION.contains("CTF") && activeChar._inEventCTF && CTF.is_started())
            msg = "Buffer is not available in CTF";
        else if (Config.BUFFER_PROTECTION.contains("DM") && activeChar._inEventDM && DM.is_started())
            msg = "Buffer is not available in DM";
		if(msg!=null)
			activeChar.sendMessage(msg);

		return msg==null;
	}
    @Override
	public String getHtmlPath(int npcId, int val)
	{
		String pom;

		if (val == 0)
			pom = "" + npcId;
		else
			pom = npcId + "-" + val;

        return "data/custom/S-Project/S-Buffer/" + pom + ".htm";
	}

    @Override
	public void onBypassFeedback(L2PcInstance player, String command)
	{
		StringTokenizer st = new StringTokenizer(command, " ");
		String actualCommand = st.nextToken(); // Get actual command

		if (actualCommand.startsWith("Chat"))
		{
			String filename = "data/custom/S-Project/S-Buffer/40001.htm";
            int cmdChoice = Integer.parseInt(command.substring(5,7).trim());
			if(cmdChoice>0)
			{
				filename = "data/custom/S-Project/S-Buffer/40001-"+cmdChoice+".htm";
			}
            NpcHtmlMessage html = new NpcHtmlMessage(1);
		    html.setFile(filename);
            sendHtmlMessage(player,html);
		    player.sendPacket(new ActionFailed());
        }
        else if(actualCommand.startsWith("Cancel"))
        {
            player.stopAllEffects();
            NpcHtmlMessage html = new NpcHtmlMessage(1);
		    html.setFile("data/custom/S-Project/S-Buffer/40001.htm");
            sendHtmlMessage(player,html);
		    player.sendPacket(new ActionFailed());
        }
        else if(actualCommand.startsWith("PetCancel"))
        {
            L2Summon pet=player.getPet();
            if(pet!=null)
            pet.stopAllEffects();
            NpcHtmlMessage html = new NpcHtmlMessage(1);
		    html.setFile("data/custom/S-Project/S-Buffer/40001-08.htm");
            sendHtmlMessage(player,html);
		    player.sendPacket(new ActionFailed());
        }
        else if(actualCommand.startsWith("Regenerate"))
        {
            player.setCurrentHpMp(player.getMaxHp(),(player.getMaxMp()));
            player.setCurrentCp(player.getMaxCp());
            NpcHtmlMessage html = new NpcHtmlMessage(1);
		    html.setFile("data/custom/S-Project/S-Buffer/40001.htm");
            sendHtmlMessage(player,html);
		    player.sendPacket(new ActionFailed());
        }
        else if(actualCommand.startsWith("PetRegenerate"))
        {
            L2Summon pet=player.getPet();
            pet.setCurrentHpMp(pet.getMaxHp(),(pet.getMaxMp()));
            pet.setCurrentCp(pet.getMaxCp());
            NpcHtmlMessage html = new NpcHtmlMessage(1);
		    html.setFile("data/custom/S-Project/S-Buffer/40001-08.htm");
            sendHtmlMessage(player,html);
		    player.sendPacket(new ActionFailed());
        }

        else if(actualCommand.startsWith("Buff"))
        {
            String filename = "data/custom/S-Project/S-Buffer/40001",v;
            int cmdChoice = Integer.parseInt(command.substring(5, 7).trim());
			int id=TableId[cmdChoice];
			int dialog=TableDialog[cmdChoice];
			int level=SkillTable.getInstance().getMaxLevel(id,0);
            if(id==4554)level=4;
            if(dialog==0)v="";
            else v="-"+Integer.toString(dialog);
            player.stopSkillEffects(id);
            SkillTable.getInstance().getInfo(id,level).getEffects(player,player);
            NpcHtmlMessage html = new NpcHtmlMessage(1);
		    html.setFile(filename+v+".htm");
            sendHtmlMessage(player,html);
		    player.sendPacket(new ActionFailed());
        }

		else if(actualCommand.startsWith("PetBuff"))
        {
            String filename = "data/custom/S-Project/S-Buffer/40001",v;
            int cmdChoice = Integer.parseInt(command.substring(5, 7).trim());
			int id=TableId[cmdChoice];
			int dialog=TableDialog[cmdChoice];
			int level=SkillTable.getInstance().getMaxLevel(id,0);
            if(dialog==0)v="";
            else v="-"+Integer.toString(dialog);
            L2Summon pet=player.getPet();
            	if(pet!=null)
					{
						pet.stopSkillEffects(id);
						SkillTable.getInstance().getInfo(id,level).getEffects(pet,pet);
                    }
                else player.sendMessage("У вас нет суммона или питомца");
            NpcHtmlMessage html = new NpcHtmlMessage(1);
		    html.setFile(filename+v+".htm");
            sendHtmlMessage(player,html);
		    player.sendPacket(new ActionFailed());
        }

        else if(actualCommand.startsWith("save"))
        {
            int cmdChoice = Integer.parseInt(command.substring(5, 6).trim());
            int flag=0;
			NpcHtmlMessage html = new NpcHtmlMessage(1);
			if(cmdChoice>3)
			{
				html.setFile("data/custom/S-Project/S-Buffer/40001-11.htm");
				flag=1;
			}
            else
			html.setFile("data/custom/S-Project/S-Buffer/40001.htm");
			CreateScheme(player,Integer.toString(cmdChoice),flag);
            sendHtmlMessage(player,html);
		    player.sendPacket(new ActionFailed());
        }
        else if(actualCommand.startsWith("give"))
        {
            int cmdChoice = Integer.parseInt(command.substring(5, 6).trim());
            if((cmdChoice<1)&&(cmdChoice>6))return;
            String key="data/custom/S-Project/S-Buffer/40001",sKey=Integer.toString(cmdChoice);
            int flag=0;
            NpcHtmlMessage html = new NpcHtmlMessage(1);

            if(cmdChoice>3)
            {
                flag=1;
                key="data/custom/S-Project/S-Buffer/40001-8";
            }

            if (CharShemesTable.getInstance().getScheme(player.getObjectId(),sKey)!=null)
            {
                if(flag==0)
                {
                    for (L2Skill sk : CharShemesTable.getInstance().getScheme(player.getObjectId(),sKey))
					{
                        player.stopSkillEffects(sk.getId());
                        sk.getEffects(this, player);
					}
                }
                else
                {
                    for (L2Skill sk : CharShemesTable.getInstance().getScheme(player.getObjectId(),sKey))
					{
                        L2Summon pet = player.getPet();
                        if(pet!=null)
                        {
                            pet.stopSkillEffects(sk.getId());
                            sk.getEffects(this, pet);
                        }
                    }
                }
                html.setFile(key+".htm");
            }
            else
            {
                player.sendMessage("Профиль "+sKey+" не найден");
                return;
            }
            sendHtmlMessage(player,html);
            player.sendPacket(new ActionFailed());
        }
        else
        {
			super.onBypassFeedback(player, command);
		}

    }
    private void sendHtmlMessage(L2PcInstance player, NpcHtmlMessage html)
	{
		html.replace("%objectId%", String.valueOf(getObjectId()));
		html.replace("%npcId%", String.valueOf(getNpcId()));
		player.sendPacket(html);
	}

    private void CreateScheme(L2PcInstance player,String name,int flag)
	{
			if (CharShemesTable.getInstance().getAllSchemes(player.getObjectId()) != null
			&& CharShemesTable.getInstance().getAllSchemes(player.getObjectId()).containsKey(name))
			{
				CharShemesTable.getInstance().getAllSchemes(player.getObjectId()).remove(name);
			}
			if (CharShemesTable.getInstance().getAllSchemes(player.getObjectId()) == null)
                {
                    CharShemesTable.getInstance().getSchemesTable().put(player.getObjectId(),
                            new FastMap<String, FastList<L2Skill>>(6));
                }
				CharShemesTable.getInstance().setScheme(player.getObjectId(),name.trim(),
						new FastList<L2Skill>(69));
             L2Effect[] s;
            if (flag==0)
            {
                 s= player.getAllEffects();

            }
			else
			{
			L2Summon pet=player.getPet();
            s=pet.getAllEffects();
			}
            int Id;
            Boolean Ok=false;
            int i = 0;
            while (i < s.length)
        {
            L2Effect value = s[i];
            Id = value.getSkill().getId();
            int k = 0;
            while (k < TableId.length)
            {
                if (Id == TableId[k])
                {
                    Ok = true;
                    break;
                }
                k++;
            }
            if (Ok)
                CharShemesTable.getInstance().getScheme(player.getObjectId(), name).add(SkillTable.getInstance().getInfo(Id, value.getSkill().getLevel()));

            Ok = false;
            i++;
        }
        player.sendMessage("Профиль "+name+" успешно сохранён");
    }
}