package net.sf.l2j.gameserver.model.actor.instance;

import net.sf.Event.CTF;
import net.sf.Event.DM;
import net.sf.Event.TvT;
import net.sf.l2j.Config;
import net.sf.l2j.gameserver.SevenSigns;
import net.sf.l2j.gameserver.instancemanager.CastleManager;
import net.sf.l2j.gameserver.model.L2Character;
import net.sf.l2j.gameserver.model.L2World;
import net.sf.l2j.gameserver.model.actor.instance.L2PcInstance;
import net.sf.l2j.gameserver.network.serverpackets.ActionFailed;
import net.sf.l2j.gameserver.network.serverpackets.NpcHtmlMessage;
import net.sf.l2j.gameserver.templates.L2NpcTemplate;

/**
 * @update S-Project
 */
public class L2CastleMagicianInstance extends L2FolkInstance
{
	protected static final int COND_ALL_FALSE = 0;
	protected static final int COND_BUSY_BECAUSE_OF_SIEGE = 1;
	protected static final int COND_OWNER = 2;

	public L2CastleMagicianInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public void showChatWindow(L2PcInstance player, int val)
    {
		player.sendPacket(new ActionFailed());
        String filename = (new StringBuilder()).append("data/").append(Config.HTML_DIR).append("/castlemagician/magician-no.htm").toString();

		int condition = validateCondition(player);
		if (condition > COND_ALL_FALSE)
        {
			if (condition == COND_BUSY_BECAUSE_OF_SIEGE)
				filename = (new StringBuilder()).append("data/").append(Config.HTML_DIR).append("/castlemagician/magician-busy.htm").toString();
			else if (condition == COND_OWNER)
			{
				if (val == 0)
                filename = (new StringBuilder()).append("data/").append(Config.HTML_DIR).append("/castlemagician/magician.htm").toString();
				else
                filename = (new StringBuilder()).append("data/").append(Config.HTML_DIR).append("/castlemagician/magician-").append(val).append(".htm").toString();
			}
		}

		NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
		html.setFile(filename);
		html.replace("%objectId%", String.valueOf(getObjectId()));
		html.replace("%npcname%", String.valueOf(getName()+" "+getTitle()));
		player.sendPacket(html);
	}

	@Override
	public void onBypassFeedback(L2PcInstance player, String command)
    {
		if (command.startsWith("Chat"))
		{
			int val = 0;
			try
			{
				val = Integer.parseInt(command.substring(5));
			}
			catch (IndexOutOfBoundsException ioobe){}
			catch (NumberFormatException nfe){}
			showChatWindow(player, val);
		}
		else if (command.equalsIgnoreCase("Clan_Gate"))
			{
				L2PcInstance leader;
				leader = (L2PcInstance) L2World.getInstance().findObject(player.getClan().getLeaderId());
                                if (leader == null)
                                        return;
				if (leader.atEvent)
				{
					player.sendMessage("Ваш КЛ на евенте.");
					return;
				}

				else if (leader.isInJail())
				{
					player.sendMessage("Ваш КЛ в тюрьме.");
					return;
				}
                if(player._inEventTvT && TvT.is_started())
		        {
			        player.sendMessage("You may not use an escape skill in TvT.");
			        return;
		        }

		        if(player._inEventCTF && CTF.is_started())
		        {
			        player.sendMessage("You may not use an escape skill in CTF.");
			        return;
		        }

		        if(player._inEventDM && DM.is_started())
		        {
			        player.sendMessage("You may not use an escape skill in DM.");
			        return;
		        }
				else if (leader.isInOlympiadMode())
				{
					player.sendMessage("Ваш КЛ сейчас на олимпиаде.");
					return;
				}

				else if (leader.inObserverMode())
				{
					player.sendMessage("Your leader is in Observer Mode.");
					return;
				}

				else  if (leader.isInDuel())
				{
					player.sendMessage("Ваш КЛ находится в дуэли.");
					return;
				}

				else if (leader.isFestivalParticipant())
				{
					player.sendMessage("Ваш КЛ на фестивале.");
					return;
				}

				else if (leader.isInParty() && leader.getParty().isInDimensionalRift())
				{
					player.sendMessage("Ваш КЛ в dimensional rift.");
					return;
				}

				else if (leader.getClan() != null && CastleManager.getInstance().getCastleByOwner(leader.getClan()) != null
						&& CastleManager.getInstance().getCastleByOwner(leader.getClan()).getSiege().getIsInProgress())
				{
					player.sendMessage("Ваш КЛ на осаде.");
					return;
				}

				int leaderx;
				int leadery;
				int leaderz;

				leaderx = leader.getX();
				leadery = leader.getY();
				leaderz = leader.getZ();

				player.teleToLocation(leaderx, leadery, leaderz);
                player.sendMessage("Вы были телепортированы к своему КЛ.");
			}
		else
			super.onBypassFeedback(player, command);
	}

	protected int validateCondition(L2PcInstance player)
	{
		if (player.isGM()) return COND_OWNER;
		if (getCastle() != null && getCastle().getCastleId() > 0)
		{
			if (player.getClan() != null)
			{
				if (getCastle().getSiege().getIsInProgress())
					return COND_BUSY_BECAUSE_OF_SIEGE;
				else if (getCastle().getOwnerId() == player.getClanId())
					return COND_OWNER;
			}
		}
		return COND_ALL_FALSE;
	}
	private static final boolean validateGateCondition(L2PcInstance clanLeader, L2PcInstance player)
	{
		if (clanLeader.isAlikeDead())
		{
			// Need retail message if there's one.
			player.sendMessage("Couldn't teleport to clan leader. The requirements was not meet.");
			return false;
		}

		if (clanLeader.isInStoreMode())
		{
			// Need retail message if there's one.
			player.sendMessage("Couldn't teleport to clan leader. The requirements was not meet.");
			return false;
		}

		if (clanLeader.isRooted() || clanLeader.isInCombat())
		{
			// Need retail message if there's one.
			player.sendMessage("Couldn't teleport to clan leader. The requirements was not meet.");
			return false;
		}

		if (clanLeader.isInOlympiadMode())
		{
			// Need retail message if there's one.
			player.sendMessage("Couldn't teleport to clan leader. The requirements was not meet.");
			return false;
		}

		if (clanLeader.isFestivalParticipant())
		{
			// Need retail message if there's one.
			player.sendMessage("Couldn't teleport to clan leader. The requirements was not meet.");
			return false;
		}

		if (clanLeader.inObserverMode())
		{
			// Need retail message if there's one.
			player.sendMessage("Couldn't teleport to clan leader. The requirements was not meet.");
			return false;
		}

		if (clanLeader.isInsideZone(L2Character.ZONE_NOSUMMONFRIEND))
		{
			// Need retail message if there's one.
			player.sendMessage("Couldn't teleport to clan leader. The requirements was not meet.");
			return false;
		}

		if (player.isIn7sDungeon())
		{
			final int targetCabal = SevenSigns.getInstance().getPlayerCabal(L2PcInstance.load(clanLeader.getObjectId()));
			if (SevenSigns.getInstance().isSealValidationPeriod())
			{
				if (targetCabal != SevenSigns.getInstance().getCabalHighestScore())
				{
					// Need retail message if there's one.
					player.sendMessage("Couldn't teleport to clan leader. The requirements was not meet.");
					return false;
				}
			}
			else
			{
				if (targetCabal == SevenSigns.CABAL_NULL)
				{
					// Need retail message if there's one.
					player.sendMessage("Couldn't teleport to clan leader. The requirements was not meet.");
					return false;
				}
			}
		}
		return true;
	}
}