/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver.model.actor.instance;

import net.sf.l2j.gameserver.ThreadPoolManager;
import net.sf.l2j.gameserver.datatables.SkillTable;
import net.sf.l2j.gameserver.model.*;
import net.sf.l2j.gameserver.network.serverpackets.MagicSkillUser;
import net.sf.l2j.gameserver.templates.L2NpcTemplate;
import net.sf.l2j.gameserver.util.Util;
import net.sf.l2j.util.Rnd;

import java.util.Collection;
import java.util.concurrent.ScheduledFuture;

/**
 * @author Drunkard Zabb0x
 * @update S-Project
 */
public class L2XmassTreeInstance extends L2NpcInstance
{
	private ScheduledFuture<?> _aiTask;

	class XmassAI implements Runnable
	{
		private L2XmassTreeInstance _caster;

		protected XmassAI(L2XmassTreeInstance caster)
		{
			_caster = caster;
		}

		public void run()
		{
			Collection<L2PcInstance> plrs = getKnownList().getKnownPlayers().values();
			{
				for (L2PcInstance player : plrs)
				{
					if(!Util.checkIfInRange(400, _caster, player, true) || player.isDead() || player.isInvul())
						continue;
					int i = Rnd.nextInt(5);
					if(i==6)
						i=1844; //Winter Holiday Festival; 6106
					handleCast(player, (4262 + i));
				}
			}
		}

		private boolean handleCast(L2PcInstance player, int skillId)
		{
			int skillLvl;
			switch(skillId)
			{
				case 4262:
					skillLvl = 2;
					break;
				case 4265:
				case 4266:
					skillLvl = 3;
					break;
				default:
					skillLvl = 1;
			}
			L2Skill skill = SkillTable.getInstance().getInfo(skillId, skillLvl);
			if (player.getFirstEffect(skill) == null)
			{
				setTarget(player);
				doCast(skill);
				MagicSkillUser msu = new MagicSkillUser(_caster, player, skill.getId(), skillLvl, skill.getHitTime(), 0);
				broadcastPacket(msu);
				return true;
			}
			return false;
		}
	}

	public L2XmassTreeInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
		_aiTask = ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new XmassAI(this), 5000, 5000);
	}

	@Override
	public void deleteMe()
	{
		if (_aiTask != null)
			_aiTask.cancel(true);
		super.deleteMe();
	}

	@Override
	public int getDistanceToWatchObject(L2Object object)
	{
		return 900;
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Object#isAttackable()
	 */
	@Override
	public boolean isAutoAttackable(L2Character attacker)
	{
		return false;
	}
}