package net.sf.l2j.gameserver.network.clientpackets;

import net.sf.l2j.*;
import net.sf.l2j.gameserver.*;
import net.sf.l2j.gameserver.communitybbs.Manager.RegionBBSManager;
import net.sf.l2j.gameserver.datatables.SkillTable;
import net.sf.l2j.gameserver.instancemanager.CustomZoneManager;
import net.sf.l2j.gameserver.model.*;
import net.sf.l2j.gameserver.model.actor.instance.L2PcInstance;
import net.sf.l2j.gameserver.network.SystemMessageId;
import net.sf.l2j.gameserver.network.serverpackets.ActionFailed;
import net.sf.l2j.gameserver.network.serverpackets.*;
import net.sf.l2j.gameserver.taskmanager.AttackStanceTaskManager;
import net.sf.olympiad.Olympiad;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.logging.Logger;

/**
 * @update S-Project
 */
public final class Logout extends L2GameClientPacket
{
	private static final String _C__09_LOGOUT = "[C] 09 Logout";
	private static Logger _log = Logger.getLogger(Logout.class.getName());

	@Override
	protected void readImpl()
	{
	}

	@Override
	protected void runImpl()
	{
		L2PcInstance player = getClient().getActiveChar();

		if (player == null)
			return;

		if (!(player.isGM()))
        {
            if(CustomZoneManager.getInstance().checkIfInZone("NoEscape", player))
            {
                player.sendPacket(SystemMessage.sendString("Это не лучшее место для выхода из игры"));
                player.sendPacket(new ActionFailed());
                return;                   
            }
        }

		player.getInventory().updateDatabase();

		if(AttackStanceTaskManager.getInstance().getAttackStanceTask(player)&& !player.isGM())
		{
			if (Config.DEBUG) _log.fine("Player " + player.getName() + " tried to logout while fighting");

			player.sendPacket(new SystemMessage(SystemMessageId.CANT_LOGOUT_WHILE_FIGHTING));
			player.sendPacket(new ActionFailed());
			return;
		}
        if (player.isFishing())
 		{
			player.sendMessage("Вы не можете выйти во время рыбалки");
 			return;
 		}
		if(player.atEvent)
        {
			player.sendPacket(SystemMessage.sendString("Вы не можете выйти, вы на евенте"));
			return;
		}
        if(player.getPet() != null) 
         {
                player.sendMessage("Вы неможете выйти из игры если питомец не отозван");
                return;
         }
         if(player.isCastingNow())
         {
                player.sendMessage("Вы неможете выйти из игры во время кастования");
                return;
         }
        if (player.isInOlympiadMode() || Olympiad.getInstance().isRegisteredInComp(player) || player.getOlympiadGameId() > 0)
        {
            player.sendMessage("Вы не можете выйти, вы на олимпиаде");
            return;
        }
        if (player.getActiveEnchantItem() != null)
		{
                player.sendMessage("Вы не можете выйти, во время точки");
				return;
		}
		if (player.isFestivalParticipant())
        {
			if (SevenSignsFestival.getInstance().isFestivalInitialized())
			{
				player.sendMessage("Вы не можете выйти, вы в пати и на Фестивале.");
				return;
			}
			L2Party playerParty = player.getParty();

			if (playerParty != null)
				player.getParty().broadcastToPartyMembers(SystemMessage.sendString(player.getName() + " has been removed from the upcoming festival."));
		}
		if (player.isFlying())
		{
			player.removeSkill(SkillTable.getInstance().getInfo(4289, 1));
		}

		RegionBBSManager.getInstance().changeCommunityBoard();

		player.deleteMe();
		notifyFriends(player);
	}

	private void notifyFriends(L2PcInstance cha)
	{
		java.sql.Connection con = null;

		try
        {
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement;
			statement = con.prepareStatement("SELECT friend_name FROM character_friends WHERE char_id=?");
			statement.setInt(1, cha.getObjectId());
			ResultSet rset = statement.executeQuery();

			L2PcInstance friend;
			String friendName;

			while (rset.next())
			{
				friendName = rset.getString("friend_name");

				friend = L2World.getInstance().getPlayer(friendName);

				if (friend != null)
				{
					friend.sendPacket(new FriendList(friend));
				}
			}

			rset.close();
			statement.close();
		}
		catch (Exception e)
        {
			_log.warning("could not restore friend data:"+e);
		}
		finally
        {
			try
            {
                con.close();
            }
            catch (Exception e)
            {
            }
		}
	}

	@Override
	public String getType()
	{
		return _C__09_LOGOUT;
	}
}