package net.sf.l2j.gameserver.util;

import javolution.util.FastMap;
import javolution.util.FastMap.Entry;
import net.sf.l2j.Config;
import net.sf.l2j.gameserver.GameTimeController;

import java.util.logging.Logger;

/**
 * new Flood protector
 * @update S-Project
 */
public class FloodProtector
{
	private static final Logger _log = Logger.getLogger(FloodProtector.class.getName());
	private static FloodProtector _instance;

	public static final FloodProtector getInstance()
	{
		if(_instance == null)
		{
			_instance = new FloodProtector();
		}

		return _instance;
	}

	// =========================================================
	// Data Field
	private static FastMap<Integer, Integer[]> _floodClient;
	// =========================================================
	// reuse delays for protected actions (in game ticks 1 tick = 100ms)
	private static final int[] REUSEDELAY = new int[]
	{
			Config.PROTECTED_BYPASS_C,
			42, //rolldice
			42, //fireworks
			Config.PROTECTED_HEROVOICE_C,
			Config.PROTECTED_MULTISELL_C,
			Config.PROTECTED_SUBCLASS_C,
			Config.GLOBAL_CHAT_DELAY,
            Config.PROTECTED_DROP_C,
            Config.PROTECTED_ENCHANT_C,
			Config.PROTECTED_UseCommand,
			Config.PROTECTED_WEREHOUSE_C,
			Config.PROTECTED_USE_ITEM_C,
            100,
            15,
            Config.PVPPK_PROTECT
	};

	// =============================================================
	public static final int PROTECTED_BYPASS = 0;
	public static final int PROTECTED_ROLLDICE = 1;
	public static final int PROTECTED_FIREWORK = 2;
	public static final int PROTECTED_HEROVOICE = 3;
	public static final int PROTECTED_MULTISELL = 4;
	public static final int PROTECTED_SUBCLASSPROTECT = 5;
    public static final int PROTECTED_CHAT = 10;
    public static final int PROTECTED_NONDUPE = 5;
    public static final int PROTECTED_ENCH = 10;
    public static final int PROTECTED_UseCommand = 11;
    public static final int PROTECTED_WH = 12;
    public static final int PROTECTED_USEITEM = 0;
    public static final int PROTECTED_SOCIAL = 9;
	public static final int PROTECTED_UNKNOWNPACKET = 6;
    public static final int PROTECTED_ITEMPETSUMMON = 3;
    public static final int PROTECTED_PVPPK = 7;
	// =============================================================
	// Constructor
	private FloodProtector()
	{
		_log.info("Load Flood Protector");
		_floodClient = new FastMap<Integer, Integer[]>(Config.FLOODPROTECTOR_INITIALSIZE).setShared(true);
	}

	/**
	 * Add a new player to the flood protector (should be done for all players when they enter the world)
	 *
	 * @param playerObjId
	 */
	public void registerNewPlayer(int playerObjId)
	{
		// create a new array
		Integer[] array = new Integer[REUSEDELAY.length];
		for(int i = 0; i < array.length; i++)
		{
			array[i] = 0;
		}

		// register the player with an empty array
		_floodClient.put(playerObjId, array);
	}

	/**
	 * Remove a player from the flood protector (should be done if player loggs off)
	 *
	 * @param playerObjId
	 */
	public void removePlayer(int playerObjId)
	{
		_floodClient.remove(playerObjId);
	}

	/**
	 * Return the size of the flood protector
	 *
	 * @return size
	 */
	public int getSize()
	{
		return _floodClient.size();
	}

	/**
	 * Try to perform the requested action
	 *
	 * @param playerObjId
	 * @param action
	 * @return true if the action may be performed
	 */
	public boolean tryPerformAction(int playerObjId, int action)
    {
          Entry<Integer, Integer[]> entry = _floodClient.getEntry(playerObjId);
          if (entry == null)
                return false;
          Integer[] value = entry.getValue();

          if (value[action] < GameTimeController.getGameTicks())
          {
                 value[action] = GameTimeController.getGameTicks()+REUSEDELAY[action];
                 entry.setValue(value);
                 return true;
          }
           return false;
     }
}