package net.sf.l2j.gameserver.network.clientpackets;

import net.sf.l2j.Config;
import net.sf.l2j.gameserver.Shutdown;
import net.sf.l2j.gameserver.model.*;
import net.sf.l2j.gameserver.model.actor.instance.*;
import net.sf.l2j.gameserver.network.SystemMessageId;
import net.sf.l2j.gameserver.network.serverpackets.SystemMessage;
import net.sf.l2j.gameserver.util.IllegalPlayerAction;
import net.sf.l2j.gameserver.util.Util;

import java.util.logging.Logger;

/**
 * @update S-Project
 */
public final class RequestGiveItemToPet extends L2GameClientPacket
{
	private static final String REQUESTCIVEITEMTOPET__C__8B = "[C] 8B RequestGiveItemToPet";
	private static Logger _log = Logger.getLogger(RequestGetItemFromPet.class.getName());

	private int _objectId;
	private int _amount;

	@Override
	protected void readImpl()
	{
		_objectId = readD();
		_amount   = readD();
	}

	@Override
	protected void runImpl()
	{
		L2PcInstance player = getClient().getActiveChar();
		PcInventory inventory = player.getInventory();

        if (player == null || player.getPet() == null || !(player.getPet() instanceof L2PetInstance)) return;

        if (Config.SAFE_REBOOT && Config.SAFE_REBOOT_DISABLE_TRANSACTION && Shutdown.getCounterInstance() != null
        		&& Shutdown.getCounterInstance().getCountdown() <= Config.SAFE_REBOOT_TIME)
        {
			player.sendMessage("Transactions are not allowed during restart/shutdown.");
			player.sendPacket(new SystemMessage(SystemMessageId.NOTHING_HAPPENED));
            return;
        }

        //Pet enchant exploit fix
        if (player.getActiveEnchantItem() != null)
        { 
                Util.handleIllegalPlayerAction(player,"Player "+player.getName()+" tried to use enchant exploit and got Banned!", IllegalPlayerAction.PUNISH_KICKBAN);
                return;
        }
        if (!Config.ALT_GAME_KARMA_PLAYER_CAN_TRADE && player.getKarma() > 0) return;
        if (player.getPrivateStoreType() != 0)
        {
            player.sendMessage("Cannot exchange items while trading");
            return;
        }

		// Exploit fix for hero weapons uses pet inventory to buy new one.
		L2ItemInstance item = player.getInventory().getItemByObjectId(_objectId);
		if (item.isAugmented())
		{
			player.sendMessage("Due To Augmentation Weapons Protection , You Can't Use Pet's Inventory");
			return;
		}

		if (item == null)
			return;

		if (item.isHeroItem())
		{
			player.sendMessage("Duo To Hero Weapons Protection u Canot Use Pet's Inventory");
			return;
		}

		if (!item.isDropable() || !item.isDestroyable() || !item.isTradeable())
		{
			sendPacket(new SystemMessage(SystemMessageId.ITEM_NOT_FOR_PETS));
			return;
		}

        L2PetInstance pet = (L2PetInstance)player.getPet();

		if (pet.isDead())
		{
			sendPacket(new SystemMessage(SystemMessageId.CANNOT_GIVE_ITEMS_TO_DEAD_PET));
			return;
		}

		if(_amount < 0)
		{
			return;
		}

		if (player.transferItem("Transfer", _objectId, _amount, pet.getInventory(), pet) == null)
		{
			_log.warning("Invalid item transfer request: " + pet.getName() + "(pet) --> " + player.getName());
		}
	}

	@Override
	public String getType()
	{
		return REQUESTCIVEITEMTOPET__C__8B;
	}
}